# -*- coding: utf-8 -*-
import os
import collections
import datetime
import xlrd
import re
import logging

from django import forms
from django.conf import settings
from django.conf.urls import patterns, url
from django.contrib import admin
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.template.defaultfilters import date
from django.http import HttpResponse, HttpResponseNotFound, HttpResponseServerError
from django.contrib.auth.models import User
from django.utils.safestring import mark_safe

from jmb.core.admin.options import ExtendibleModelAdmin
from jmb.newsletter2.models import Newsletter

from mailup.clients import MailUpClientSingleton
from mailup.exceptions import GroupNotFoundException, RecipientNotFoundException, MailUpException
from mailup.providers import MailUpComponentProvider

from acomea.contact.utils.mailer import get_queryset

from acomea.contact.tasks import (
    SendNewLetterTask, SendNewLetter2Task, SendInviteTask, SendBonificaTask, export_csv_async
)

from acomea.contact.forms import (
    SetNewsletterForm, SetNewsletter2Form, SetInviteForm, SetBonificaForm, SetInviteNoControlsForm
)

from acomea.contact.models import (
    Role, Title, Category, Group, Interest,
    Product, Network, Contact, Email, Phone, WebProfile,
    PhoneCategory, CategoryProfile, AddressScope, Address,
    AcomeaEmailObject, ContactGroup,
    Campaign, Source, ContactCampaignSource)
from acomea.contact.extended_mailer import Mailer
from acomea.contact.utils.importation import deep_import_dispatcher
# from acomea.contact.utils.importation import deep_import_dispatcher2
from acomea.contact.forms import SetNewsletterExtraForm, JoinToGroupForm, SetNewsletter2ExtraForm

from acomea.contact.utils.mailer import test_smtp_connection
from acomea.contact.filters import FilterSet
# from jmb.filters.filterset import FilterSet

task_logger = logging.getLogger('celery.task')

class EmailInline(admin.TabularInline):
    model = Email
    extra = 0
    fields = ('status', 'email', 'default', 'note',)


class PhoneInline(admin.TabularInline):
    model = Phone
    extra = 0
    fields = ('status', 'category', 'phone', 'default', 'note',)


class WebProfileInline(admin.TabularInline):
    model = WebProfile
    extra = 0
    fields = ('category', 'name', 'note',)


class AddressInline(admin.StackedInline):
    model = Address
    extra = 0
    fieldsets = (
        (None, {
            'fields': (
                'status',
                'address_scopes',
                ('address_type', 'name', 'number',),
                ('zip_code', 'locality', 'province'),
                ('region', 'country')
            ),
        }),
    )


class ContactGroupInline(admin.TabularInline):
    model = ContactGroup
    extra = 0
    fields = ('group', 'date_create')
    readonly_fields = ('group', 'date_create')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    # def get_queryset(self, request):
    #     return super(ContactGroupInline, self).get_queryset(request).order_by('date_create')


class ContactCampaignSourceInline(admin.TabularInline):
    model = ContactCampaignSource
    extra = 0
    fields = ('contact', 'campaign', 'source', 'date_create')
    readonly_fields = ('contact', 'campaign', 'source', 'date_create')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class CustomExtendibleModelAdmin(ExtendibleModelAdmin):

    def get_filterset_class(self, request):
        """Return a filterset_class suitable for advanced search
        first look if attribute ``advanced_search_fields``
        """
        fields = self.get_advanced_search_fields(request)

        if fields:
            meta = type('Meta', (object,), {'fields': fields, 'model': self.model})
            AdminFilterset = type('Admin%sFilterset' % self.model.__name__,
                                  (FilterSet,),
                                  {'Meta': meta, 'fields': fields})
            return AdminFilterset
        # elif hasattr(self, 'get_filterset_class'):
        #     return self.get_filterset()
        return None


class ContactAdmin(CustomExtendibleModelAdmin):
    class Media:
        css = {
            "all": (
                "contact/css/admin_contact.css",
            )
        }

    list_display = (
        'id', 'get_status_icon', 'get_string_groups', 'title', 'role', 'first_name', 'last_name', 'user',
        'organization', 'origin', 'get_string_phones', 'get_string_emails',
        'is_test_contact', 'auth_trattamento_dati', 'auth_materiale_commerciale',
        'auth_societa_terze', 'auth_premium', 'auth_profilazione',
        'bonifica',
        'mailup_recipient_id', 'date_create', 'date_last_modify', 'date_last_modify_privacy',
        'first_group', 'date_group_join',
        'trace_request_plugin',
        'get_edit_icon', 'get_delete_icon',
    )
    list_display_csv = (
        'get_title_id', 'get_first_name_title', 'get_last_name_title', 'birth_date', 'ssn', 'organization', 'origin',
        'get_string_group_ids',
        'get_role_id', 'url', 'source', 'get_category_id', 'get_string_interest_ids', 'get_string_product_ids',
        'get_network_id',
        'date_subscription', 'get_is_test_contact_display',
        'get_default_email', 'get_email1', 'get_email2',
        'get_mobile', 'get_home_phone', 'get_work_phone',
        'get_address_scope_id', 'get_address_type', 'get_address_name', 'get_address_number', 'get_address_zip_code',
        'get_address_locality', 'get_address_province', 'get_address_region', 'get_address_country_id',
        'note',
        'get_auth_trattamento_dati', 'get_auth_materiale_commerciale', 'get_auth_societa_terze', 'get_auth_premium',
        'get_auth_profilazione',
        'get_bonifica',
        'get_date_last_modify_privacy', 'date_create', 'date_last_modify', 'date_last_modify_privacy',
        'date_group_join', 'trace_request_plugin',
    )
    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (_('Groups'), {
            'fields': ('groups',),
            'classes': ('collapse',),
        }),
        (None, {
            'fields': (
                'status',
                ('title', 'role',),
                ('first_name', 'last_name',),
                'birth_date',
                'ssn',
                ('organization', 'origin'),
                ('url', 'source',),
                ('category', 'network',),
                'first_group',
            ),
        }),
        (_('Interests'), {
            'fields': ('interests',),
            'classes': ('collapse',),
        }),
        (_('Products'), {
            'fields': ('products',),
            'classes': ('collapse',),
        }),
        (_('Autorizzazioni'), {
            'fields': (
                'user',
                'bonifica',
                ('date_last_modify_privacy',),
                ('auth_trattamento_dati', 'auth_materiale_commerciale',),
                ('auth_societa_terze', 'auth_premium', 'auth_profilazione'),
            ),
            'classes': ('collapse',),
        }),
        (_('Newsletter'), {
            'fields': (
                # ('date_subscription', 'date_unsubscription',),
                'mailup_recipient_id',
                'is_test_contact',
                'date_last_modify_for_mailup',
            ),
            'classes': ('collapse',),
        }),
        (_('Trace Request'), {
            'fields': (
                'trace_request_plugin',
            ),
            'classes': ('collapse',),
        }),
        (_('Note'), {
            'fields': ('note',),
            'classes': ('collapse',),
        })
    )
    inlines = (
        ContactGroupInline,
        ContactCampaignSourceInline,
        EmailInline,
        PhoneInline,
        WebProfileInline,
        AddressInline,
    )
    list_filter = ('status', )
    advanced_search_fields = (
        ('groups',),
        ('first_group__in',),
        ('user__isnull:senza utenza',),
        ('date_create__gte:Date create from', 'date_create__lte:Date create to'),
        ('date_last_modify__gte:Date last modify from', 'date_last_modify__lte:Date last modify to'),
        ('first_name__istartswith', 'last_name__istartswith'),
        ('emails__email__icontains',),
        ('ssn__icontains',),
        ('source__icontains',),
        ('organization__icontains',),
        ('role__in', 'category__in',),
        ('interests__in',),
        ('products__in', 'network__in',),
        ('auth_trattamento_dati', 'auth_materiale_commerciale', 'auth_profilazione'),
        ('auth_societa_terze', 'auth_premium',),
        ('bonifica',),
        ('contacts_campaign_source__campaign:Campaign', 'contacts_campaign_source__source:Source',),
        #        ('date_subscription__gte', 'date_subscription__lte'),
        #        ('date_unsubscription__gte', 'date_unsubscription__lte'),
        #        ('date_unsubscription__isnull', 'is_test_contact',),
        #        ('unsubscriptions', 'is_test_contact'),
        ('note__icontains',),
        ('trace_request_plugin:Traccia richieste',),
    )

    filter_horizontal = ('groups', 'interests', 'products')
    readonly_fields = ('date_last_modify_privacy', 'trace_request_plugin', 'first_group')

    def get_form(self, request, obj=None, **kwargs):
        form = super(ContactAdmin, self).get_form(request, obj, **kwargs)
        form.base_fields['user'] = forms.ModelChoiceField(
            queryset=User.objects.all().order_by('email'),
            required=False
        )
        return form

    def changelist_view(self, request, extra_context=None):
        self.request_get_group = request.GET["groups"] if "groups" in request.GET and len(
            request.GET.getlist('groups')
        ) == 1 else None
        return super(ContactAdmin, self).changelist_view(request, extra_context=extra_context)

    def date_group_join(self, contact):
        if self.request_get_group:
            try:
                contact_group = ContactGroup.objects.get(contact=contact, group__id=self.request_get_group)
                return contact_group.date_create.strftime("%d/%m/%Y %H:%M")
            except ContactGroup.DoesNotExist:
                pass
        return ""

    date_group_join.short_description = _('Date group join')

    def get_title_id(self, obj):
        if obj.title:
            return obj.title.id
        return ""

    def get_role_id(self, obj):
        if obj.role:
            return obj.role.id
        return ""

    def get_category_id(self, obj):
        if obj.category:
            return obj.category.id
        return ""

    def get_network_id(self, obj):
        if obj.network:
            return obj.network.id
        return ""

    def get_is_test_contact_display(self, obj):
        if obj.is_test_contact:
            return True
        return False

    def get_mobile(self, obj):
        try:
            return obj.phones.filter(category__id=1, status=1)[0].phone
        except:
            return ""

    def get_home_phone(self, obj):
        try:
            return obj.phones.filter(category__id=2, status=1)[0].phone
        except:
            return ""

    def get_work_phone(self, obj):
        try:
            return obj.phones.filter(category__id=3, status=1)[0].phone
        except:
            return ""

    def get_auth_trattamento_dati(self, obj):
        return obj.auth_trattamento_dati

    def get_auth_materiale_commerciale(self, obj):
        return obj.auth_materiale_commerciale

    def get_auth_societa_terze(self, obj):
        return obj.auth_societa_terze

    def get_auth_premium(self, obj):
        return obj.auth_premium

    def get_auth_profilazione(self, obj):
        return obj.auth_profilazione

    def get_bonifica(self, obj):
        return obj.bonifica

    def get_address_scope_id(self, obj):
        return obj.get_first_address()['get_address_scope_id']

    def get_address_type(self, obj):
        return obj.get_first_address()['get_address_type']

    def get_address_name(self, obj):
        return obj.get_first_address()['get_address_name']

    def get_address_number(self, obj):
        return obj.get_first_address()['get_address_number']

    def get_address_zip_code(self, obj):
        return obj.get_first_address()['get_address_zip_code']

    def get_address_locality(self, obj):
        return obj.get_first_address()['get_address_locality']

    def get_address_province(self, obj):
        return obj.get_first_address()['get_address_province']

    def get_address_region(self, obj):
        return obj.get_first_address()['get_address_region']

    def get_address_country_id(self, obj):
        return obj.get_first_address()['get_address_country_id']

    def get_date_last_modify_privacy(self, obj):
        if obj.date_last_modify_privacy:
            return obj.date_last_modify_privacy.strftime("%d/%m/%Y")
        return ""

    def save_related(self, request, form, formsets, change):
        try:
            contact = form.instance
            previous_group_ids = list(contact.groups.all().values_list('pk', flat=True))
            super(ContactAdmin, self).save_related(request, form, formsets, change)

            new_group_ids = list(contact.groups.all().values_list('pk', flat=True))
            group_ids_to_remove = [group_id for group_id in previous_group_ids if group_id not in new_group_ids]
            group_ids_to_add = [group_id for group_id in new_group_ids if group_id not in previous_group_ids]

            # mailup client
            mailup_client = MailUpClientSingleton(
                client_id=settings.ACOMEA_MAILUP_CLIENT_ID,
                client_secret=settings.ACOMEA_MAILUP_SECRET,
                username=settings.ACOMEA_MAILUP_USERNAME,
                password=settings.ACOMEA_MAILUP_PASSWORD,
            )

            # mailup provider
            provider = MailUpComponentProvider(client=mailup_client)
            default_aca_list = provider.get_list(settings.ACOMEA_MAILUP_LIST_ID)

            contact_email = contact.get_default_email()
            if not contact_email:
                messages.warning(
                    request,
                    'Contact has not default email, mailup will not be updated'
                )
                return

            try:
                mailup_recipient = provider.get_recipient(default_aca_list.id, email=contact_email)

                # ADD RECIPIENT IN NEW GROUP
                for group_id in group_ids_to_add:
                    group = Group.objects.get(pk=group_id)
                    group_name = group.name

                    if group.mailup_group_id:
                        try:
                            mailup_group = provider.get_group(default_aca_list.id, group.mailup_group_id)
                        except GroupNotFoundException:
                            messages.error(
                                request,
                                'Group "{group_name}" with id {group_id} not found in MailUp'.format(
                                    group_name=group_name,
                                    group_id=group.mailup_group_id,
                                )
                            )
                            continue
                    else:
                        # A new group is created on mailup
                        mailup_group = provider.create_group(
                            {
                                'Name': group_name,
                                'idList': default_aca_list.id,
                                'Notes': 'Created from admin',
                            }
                        )
                        group.mailup_group_id = mailup_group.id
                        group.save()
                        messages.warning(
                            request,
                            'A new group "{group_name}" has been created in Mailup'.format(
                                group_name=mailup_group.name,
                            )
                        )

                    # Recipient is inserted in mailup group
                    mailup_group.insert_recipient(mailup_recipient.id)
                    messages.success(
                        request,
                        'Recipient "{email}" has been added to group "{group_name}" in MailUp'.format(
                            email=contact_email,
                            group_name=mailup_group.name,
                        )
                    )

                # REMOVE RECIPIENT FROM GROUP
                for group_id in group_ids_to_remove:
                    group = Group.objects.get(pk=group_id)
                    group_name = group.name
                    if group.mailup_group_id:
                        try:
                            mailup_group = provider.get_group(default_aca_list.id, group.mailup_group_id)
                        except GroupNotFoundException:
                            messages.error(
                                request,
                                'Group "{group_name}" with id {group_id} not found in MailUp'.format(
                                    group_name=group_name,
                                    group_id=group.mailup_group_id,
                                )
                            )
                            continue
                        mailup_group.extract_recipient(mailup_recipient.id)
                        messages.success(
                            request,
                            'Recipient "{email}" has been removed from group "{group_name}" in MailUp'.format(
                                email=contact_email,
                                group_name=group_name,
                            )
                        )

            except RecipientNotFoundException:
                messages.warning(
                    request,
                    "The recipient {email} was not included in any group because the recipient is not present in list".format(
                        email=contact_email,
                    )
                )
        except MailUpException:
            messages.error(
                request,
                'Error during MailUp Synchronization'
            )

    # ACTIONS
    actions = [
        'set_as_test',
        'unsubscribe_subscribers',
        'send_bonifica',
        'send_newsletter',
        #'send_newsletter2',
        'send_invite',
        'send_invite_no_controls',
        export_csv_async,
    ]

    def set_as_test(self, request, queryset):
        """
        Set contact as test
        """
        for contact in queryset:
            try:
                contact.is_test_contact = True
                contact.save()
                messages.info(request, _('%s succesfully set as test.') % contact)
            except Exception, e:
                messages.error(request, _('Error: {0} - {1}').format(contact, e))

    set_as_test.short_description = _("Set Contacts as test")

    def unsubscribe_subscribers(self, request, queryset):
        """
        Unsubscribe subscribers
        """
        now = datetime.datetime.now()
        totals = queryset.count()
        queryset = queryset.filter(date_unsubscription__isnull=True)
        changing = queryset.count()
        changed = 0
        error = 0
        already_unsubscribed = totals - changing

        messages.error(request, _("Subscribers already unsubscribed: %s") % already_unsubscribed)
        for contact in queryset:
            try:
                contact.date_unsubscription = now
                contact.save()
                changed += 1
            except Exception, e:
                error += 1
                messages.error(request, _('Error: {0} - {1}').format(contact, e))
        messages.info(request, _("Subscribers unsubscripted: %s") % changed)
        messages.error(request, _("Total errors: %s") % error)

    unsubscribe_subscribers.short_description = _("Unsubscribe subscribers")

    ####################################################################################################################
    # Celery ###########################################################################################################
    ####################################################################################################################
    def send_bonifica(self, request, queryset):
        """
        action per invio newsletter a contatti selezionati tramite task asincrono
        """
        form = None
        queryset = get_queryset(self, request, queryset)
        if request.POST.get('apply'):
            form = SetBonificaForm(request.POST)
            if form.is_valid():
                if request.POST['apply'] == 'Send Bonifica Email':
                    test_smtp_connection(request)
                    GET = request.GET.copy()
                    REQ = request.REQUEST.copy()
                    pks = request.POST.getlist('_selected_action')
                    select_across = int(request.POST.get('select_across'))
                    msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
                    messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
                    res = SendBonificaTask.delay_or_fail(
                        app_label='contact',
                        model=self.model,
                        pks=pks,
                        select_across=select_across,
                        get=GET,
                        req=REQ,
                    )
        if not form:
            form = SetBonificaForm(
                initial={
                    '_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME),
                    'select_across': request.POST.get('select_across'),
                }
            )
        context = {'contacts': queryset, 'form': form, }
        return render_to_response(
            'admin/contact/set_bonifica.html',
            context, RequestContext(request)
        )

    send_bonifica.short_description = _("Send bonifica task")

    def send_newsletter(self, request, queryset):
        """
        action per invio newsletter a contatti selezionati tramite task asincrono
        """
        form = None
        queryset = get_queryset(self, request, queryset)
        if request.POST.get('apply'):
            form = SetNewsletterForm(request.POST)
            if form.is_valid():
                test = True if request.POST['apply'] == 'Send in Test' else False
                test_smtp_connection(request)
                GET = request.GET.copy()
                REQ = request.REQUEST.copy()
                pks = request.POST.getlist('_selected_action')
                select_across = int(request.POST.get('select_across'))
                msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
                messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
                res = SendNewLetterTask.delay_or_fail(
                    app_label='contact',
                    model=self.model,
                    pks=pks,
                    select_across=select_across,
                    get=GET,
                    req=REQ,
                    newsletter=form.cleaned_data['newsletter'],
                    test=test,
                    check_can_create=True,
                )
        if not form:
            form = SetNewsletterForm(
                initial={
                    '_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME),
                    'select_across': request.POST.get('select_across'),
                }
            )
        context = {'contacts': queryset, 'form': form}
        return render_to_response(
            'admin/contact/set_newsletter.html',
            context, RequestContext(request)
        )

    send_newsletter.short_description = _("Send  newsletter task")

    def send_newsletter2(self, request, queryset):
        """
        action per invio newsletter2 a contatti selezionati tramite task asincrono
        """
        form = None
        queryset = get_queryset(self, request, queryset)
        if request.POST.get('apply'):
            form = SetNewsletter2Form(request.POST)
            if form.is_valid():
                test = True if request.POST['apply'] == 'Send in Test' else False
                test_smtp_connection(request)
                GET = request.GET.copy()
                REQ = request.REQUEST.copy()
                pks = request.POST.getlist('_selected_action')
                select_across = int(request.POST.get('select_across'))
                msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
                messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
                res = SendNewLetter2Task.delay_or_fail(
                    app_label='contact',
                    model=self.model,
                    pks=pks,
                    select_across=select_across,
                    get=GET,
                    req=REQ,
                    newsletter=form.cleaned_data['newsletter'],
                    test=test,
                )
        if not form:
            form = SetNewsletter2Form(
                initial={
                    '_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME),
                    'select_across': request.POST.get('select_across'),
                }
            )
        context = {'contacts': queryset, 'form': form}
        return render_to_response(
            'admin/contact/set_newsletter2.html',
            context, RequestContext(request)
        )

    send_newsletter2.short_description = _("Send  newsletter2 task")

    def send_invite(self, request, queryset):
        """
        action per invio invite a contatti selezionati tramite task asincrono
        """
        form = None
        queryset = get_queryset(self, request, queryset)
        if request.POST.get('apply'):
            form = SetInviteForm(request.POST)
            if form.is_valid():
                test = True if request.POST['apply'] == 'Send in Test' else False
                test_smtp_connection(request)
                GET = request.GET.copy()
                REQ = request.REQUEST.copy()
                pks = request.POST.getlist('_selected_action')
                select_across = int(request.POST.get('select_across'))
                msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
                messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
                task_logger.info('task_id: %r' % pks)
                res = SendInviteTask.delay_or_fail(
                    app_label='contact',
                    model=self.model,
                    pks=pks,
                    select_across=select_across,
                    get=GET,
                    req=REQ,
                    event=form.cleaned_data['event'],
                    test=test,
                    date_from=form.cleaned_data.get('send_from_date', None),
                    date_to=form.cleaned_data.get('send_to_date', None)
                )
        if not form:
            form = SetInviteForm(
                initial={
                    '_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME),
                    'select_across': request.POST.get('select_across'),
                }
            )
        context = {'contacts': queryset, 'form': form}
        return render_to_response(
            'admin/contact/send_invite.html',
            context, RequestContext(request)
        )

    send_invite.short_description = _("Send  invite task")

    def send_invite_no_controls(self, request, queryset):
        """
        action per invio invite a contatti selezionati tramite task asincrono
        """
        form = None
        queryset = get_queryset(self, request, queryset)
        if request.POST.get('apply'):
            form = SetInviteForm(request.POST)
            if form.is_valid():
                test = True if request.POST['apply'] == 'Send in Test' else False
                test_smtp_connection(request)
                GET = request.GET.copy()
                REQ = request.REQUEST.copy()
                pks = request.POST.getlist('_selected_action')
                select_across = int(request.POST.get('select_across'))
                msg = _('The task has been queued: you can monitor progress <a href="%s">here<a>')
                messages.info(request, mark_safe(msg % reverse('admin:async_task_changelist')))
                res = SendInviteTask.delay_or_fail(
                    app_label='contact',
                    model=self.model,
                    pks=pks,
                    select_across=select_across,
                    get=GET,
                    req=REQ,
                    event=form.cleaned_data['event'],
                    test=test,
                    check_can_create=False,
                    date_from=form.cleaned_data.get('send_from_date', None),
                    date_to=form.cleaned_data.get('send_to_date', None)
                )
        if not form:
            form = SetInviteForm(
                initial={
                    '_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME),
                    'select_across': request.POST.get('select_across'),
                }
            )
        context = {'contacts': queryset, 'form': form}
        return render_to_response(
            'admin/contact/send_invite_no_controls.html',
            context, RequestContext(request)
        )

    send_invite_no_controls.short_description = _("Send  invite task (NO CONTROLS)")

    def send_newsletter_extra(self, request):
        """
        Sends newsletter to extra contacts inserted
        """
        opts = self.model._meta

        if request.method == 'POST':
            form = SetNewsletterExtraForm(request.POST)
            if form.is_valid():
                test_smtp_connection(request)
                create = 0
                not_create = 0

                newsletter = form.cleaned_data['newsletter']
                emails = form.cleaned_data['emails'].split(';')
                mailer = Mailer(newsletter, test=False, is_newsletter=True)
                for email in emails:
                    context = dict(newsletter=newsletter)
                    mail_created, email_object = mailer.create_message_from_email(email, context)
                    if mail_created:
                        mailer.send(email_object)
                        create += 1
                    else:
                        not_create += 1
                        messages.error(request, _("{0}: not create {1}").format(email, email_object))

                # PRINT MESSAGES
                if create == 0:
                    messages.error(request, _('%s not create') % newsletter)
                if create > 0:
                    messages.info(request, _('{0} succesfully create to {1} emails').format(
                        newsletter, create)
                    )
                if not_create > 0:
                    messages.error(request, _('{0}: {1} in error').format(newsletter, not_create))
                return HttpResponseRedirect(request.get_full_path())
        else:
            form = SetNewsletterExtraForm()

        context = {
            'form': form,
            'opts': opts,
            'app_label': opts.app_label
        }
        return render_to_response(
            'admin/contact/set_newsletter_extra.html',
            context, RequestContext(request)
        )

    def send_newsletter2_extra(self, request):
        """
        send_newsletter_extra for jmb.newsletter2
        """
        opts = Newsletter._meta
        if request.method == 'POST':
            form = SetNewsletter2ExtraForm(request.POST)
            if form.is_valid():
                test_smtp_connection(request)
                create = 0
                not_create = 0

                newsletter = form.cleaned_data['newsletter']
                emails = form.cleaned_data['emails'].split(';')
                mailer = Mailer(newsletter, test=False, is_newsletter2=True)
                for email in emails:
                    context = dict(newsletter=newsletter)
                    mail_created, email_object = mailer.create_message_from_email(email, context)
                    if mail_created:
                        mailer.send(email_object)
                        create += 1
                    else:
                        not_create += 1
                        messages.error(request, _("{0}: not create {1}").format(email, email_object))

                # PRINT MESSAGES
                if create == 0:
                    messages.error(request, _('%s not create') % newsletter)
                if create > 0:
                    messages.info(request, _('{0} succesfully create to {1} emails').format(
                        newsletter, create)
                    )
                if not_create > 0:
                    messages.error(request, _('{0}: {1} in error').format(newsletter, not_create))
                return HttpResponseRedirect(request.get_full_path())
        else:
            form = SetNewsletter2ExtraForm()

        context = {
            'form': form,
            'opts': opts,
            # 'root_path': self.admin_site.root_path,
            'app_label': opts.app_label
        }
        return render_to_response(
            'admin/contact/set_newsletter2_extra.html',
            context, RequestContext(request)
        )

    def get_privacy_log(self, request):
        """
        return privacy log file
        """
        logfile = None
        try:
            filepath = settings.LOGGING['handlers']['privacy_log_file']['filename']
            if os.path.exists(filepath):
                logfile = open(filepath, "rb")
                filename = os.path.basename(filepath)
                filesize = os.path.getsize(filepath)
                response = HttpResponse(logfile, mimetype="text/plain")
                response['Content-Disposition'] = 'attachment; filename=%s_%s.log' % (
                    filepath.split("/")[-1], date(datetime.datetime.now(), "d-m-Y_Hi")
                )
                response['Content-Length'] = filesize
            else:
                return HttpResponseNotFound()
        except AttributeError:
            return HttpResponseServerError()
        except KeyError:
            return HttpResponseServerError()
        except IOError:
            return HttpResponseServerError()

        return response

    ## URLS
    def get_urls(self):
        urls = super(ContactAdmin, self).get_urls()
        my_urls = patterns(
            '',
            url(
                r'^base_import/$',
                self.admin_site.admin_view(self.base_importation),
                name='contact_contact_base_import'
            ),
            url(
                r'^deep_import/$',
                self.admin_site.admin_view(self.deep_importation),
                name='contact_contact_deep_import'
            ),
            url(
                r'^send_newsletter_extra/$',
                self.admin_site.admin_view(self.send_newsletter_extra),
                name='contact_contact_send_newsletter_extra'
            ),
            url(
                r'^send_newsletter2_extra/$',
                self.admin_site.admin_view(self.send_newsletter2_extra),
                name='contact_contact_send_newsletter2_extra'
            ),
            url(
                r'^get_privacy_log/$',
                self.admin_site.admin_view(self.get_privacy_log),
                name='contact_contact_get_privacy_log'
            ),

        )
        return my_urls + urls

    def deep_importation(self, request):
        """
        Import contacts from a CSV or Excel file
        """
        opts = self.model._meta
        allowed_file_extensions = ['csv', 'xls']

        if request.FILES:
            source = request.FILES.get('source')
            file_type = os.path.splitext(source.name)[1].lstrip('.')
            if file_type in allowed_file_extensions:
                inserted, updated, discarded, errors, warnings, debug_cases, updated_on_db_no_user_on_sso = deep_import_dispatcher(source, file_type)
                # inserted, updated, discarded, errors, warnings, debug_cases = deep_import_dispatcher2(source, file_type)
            else:
                error = 'Formato "%s" non supportato. Salvare il file in uno dei seguenti formati: %s' % (
                    file_type, ",".join(allowed_file_extensions)
                )
                messages.error(request, error)
                context = {
                    'title': _('Contact importation'),
                    'opts': opts,
                    'app_label': opts.app_label
                }

                return render_to_response(
                    'admin/contact/contact_deep_import.html', context, RequestContext(request)
                )

            if errors:
                for error in errors:
                    messages.error(request, error)

            messages.info(request, _('%s contacts inserted.') % inserted)
            messages.info(request, _('%s contacts  updated.') % updated)
            if updated_on_db_no_user_on_sso:
                for row in updated_on_db_no_user_on_sso:
                    messages.warning(request, u'Updated without in SSO: {}'.format(row))
            if discarded > 0:
                messages.error(request, _('%s contacts discarded!!!') % discarded)
            if warnings:
                messages.warning(request, "Warnings:\n%r" % warnings)

        # vitto: commentato - va in errore con 1.4
        context = {
            'title': _('Contact importation'),
            'opts': opts,
            #            'root_path': self.admin_site.root_path,
            'app_label': opts.app_label
        }

        return render_to_response(
            'admin/contact/contact_deep_import.html',
            context, RequestContext(request)
        )

    def base_importation(self, request):
        """
        Import contacts from a CSV or Excel file
        """
        opts = self.model._meta
        if request.FILES:
            source = request.FILES.get('source')
            file_type = os.path.splitext(source.name)[1].lstrip('.')
            if file_type in ["xls", "xlsx"]:
                if isinstance(source, basestring):
                    source = open(source, 'r')
                wb = xlrd.open_workbook(file_contents=source.read())
                sh = wb.sheet_by_index(0)

                email_list = []

                for row in range(1, sh.nrows):
                    last_name = ("%s" % sh.cell(row, 0).value).strip().title()
                    first_name = ("%s" % sh.cell(row, 1).value).strip().title()
                    organization = ("%s" % sh.cell(row, 2).value).strip()
                    email = ("%s" % sh.cell(row, 3).value).strip()
                    origin = ("%s" % sh.cell(row, 4).value).strip()
                    phone = ("%s" % sh.cell(row, 5).value).strip()
                    groups = ("%s" % sh.cell(row, 6).value).strip().split(",")

                    try:
                        validate_email(email)

                        email_list.append(email.lower())

                        if re.match("^\d+?\.\d+?$", organization) is not None:
                            organization = str(int(float(organization)))
                        if re.match("^\d+?\.\d+?$", phone) is not None:
                            phone = str(int(float(phone)))

                        emails = Email.objects.filter(email__iexact=email)

                        if emails.count():
                            contact = emails[0].contact
                        else:
                            contact = Contact(
                                first_name=first_name,
                                last_name=last_name,
                                organization=organization,
                                origin=origin
                            )
                            contact.save()
                            email_object = Email(contact=contact, email=email, default=True)
                            email_object.save()
                            if phone != "":
                                phone_object = Phone(contact=contact, phone=phone)
                                phone_object.save()

                        for group_id in groups:
                            if re.match("^\d+?\.\d+?$", group_id) is not None:
                                group_id = int(float(group_id))
                            group = Group.objects.get(id=group_id)
                            contact.groups.add(group)  # signal "contact_group_signal" change "first_group" field, to use contact instace remember to reload
                    except ValidationError as e:
                        messages.error(request, "Indirizzo %s non valido: %s" % (email, e))

                messages.success(request, "Importazione completata")
                if len(email_list):
                    messages.warning(request, "Contatti duplicati nel file: %s" % ", ".join(
                        [x for x, y in collections.Counter(email_list).items() if y > 1])
                    )
            else:
                messages.error(request, "Formato file non supportato.")

        context = {
            'title': _('Contacts base import'),
            'opts': opts,
            'app_label': opts.app_label
        }

        return render_to_response(
            'admin/contact/contact_base_import.html',
            context, RequestContext(request)
        )


class RoleAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'name', 'is_show_in_registration',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'name', 'show_in_registration'),
        }),
    )


class TitleAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'abbreviation', 'name',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'abbreviation', 'name',),
        }),
    )


class CategoryAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'name',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'name',),
        }),
    )


class GroupAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'name', 'mailup_group_id',
        'get_edit_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'name', 'mailup_group_id'),
        }),
    )
    search_fields = ['name', ]
    readonly_fields = ('mailup_group_id',)

    def delete_model(self, request, obj):
        # mailup client
        mailup_client = MailUpClientSingleton(
            client_id=settings.ACOMEA_MAILUP_CLIENT_ID,
            client_secret=settings.ACOMEA_MAILUP_SECRET,
            username=settings.ACOMEA_MAILUP_USERNAME,
            password=settings.ACOMEA_MAILUP_PASSWORD,
        )

        # mailup provider
        provider = MailUpComponentProvider(client=mailup_client)
        default_aca_list = provider.get_list(settings.ACOMEA_MAILUP_LIST_ID)
        if obj.mailup_group_id:
            mailup_group = provider.get_group(default_aca_list.id, obj.mailup_group_id)
            mailup_group.delete()
            group_name = mailup_group.name
            messages.success(
                request,
                'Group "{group_name}" with id {group_id} has been removed from MailUp'.format(
                    group_name=group_name,
                    group_id=obj.mailup_group_id,
                )
            )
        super(GroupAdmin, self).delete_model(request, obj)


class InterestAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'name',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'name',),
        }),
    )


class ProductAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'name',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'name',),
        }),
    )


class NetworkAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'name',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'name',),
        }),
    )


class EmailAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'contact', 'email', 'default',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'contact', 'email', 'default'),
        }),
    )
    readonly_fields = ('contact',)


class PhoneCategoryAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'name',
        'get_edit_icon', 'get_delete_icon'
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'name',),
        }),
    )


class PhoneAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'contact', 'phone', 'category', 'default',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'contact', 'phone', 'category', 'default'),
        }),
    )
    readonly_fields = ('contact',)


class CategoryProfileAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'name',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'name',),
        }),
    )


class WebProfileAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'contact', 'name', 'category',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'contact', 'name', 'category',),
        }),
    )
    readonly_fields = ('contact',)


class AddressScopeAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon', 'name',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('status', 'name',),
        }),
    )


class AddressAdmin(ExtendibleModelAdmin):
    list_display = (
        'id', 'get_status_icon',
        'get_string_scopes', 'get_address_type_display', 'name', 'number',
        'zip_code', 'locality', 'province', 'region', 'country',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'status',
                ('address_scopes', 'address_type',),
                ('name', 'number',),
                ('zip_code', 'locality', 'province',),
                ('region', 'country',),
            ),
        }),
    )

    filter_horizontal = ('address_scopes',)


class AcomeaEmailObjectAdmin(ExtendibleModelAdmin):
    list_display_links = []
    list_display = (
        'get_email_address', 'get_string_groups',
        'get_status', 'date_create_newsletter', 'get_subject',
    )
    advanced_search_fields = (
        ('email_object__status', 'email_object__object_id'),
        ('email_object__mailing_status__status', 'date_create_newsletter',),
    )
    readonly_fields = ('email_object',)

    def lookup_allowed(self, key, value):
        return True

    def get_email_address(self, obj):
        return obj.email_object.email_address

    def get_status(self, obj):
        return obj.email_object.get_status_display()

    def get_subject(self, obj):
        return obj.email_object.subject

    ## ACTIONS
    actions = ['join_to_group']

    def get_actions(self, request):
        actions = super(AcomeaEmailObjectAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions

    def join_to_group(self, request, queryset):
        """
        Join objects to a group
        """
        form = None

        joined = 0
        errors = []

        # verifies that all items are selected
        if request.POST.get('select_across') == '1':
            search_filterset = self.search_filterset_class(request.GET)
            queryset = search_filterset.qs

        #        queryset = queryset.distinct('email_object')

        if request.POST.get('apply'):
            form = JoinToGroupForm(request.POST)
            if form.is_valid():
                group = form.cleaned_data['group']
                for acomea_email_obj in queryset:
                    try:
                        contact = Contact.objects.get(emails__email=acomea_email_obj.email_object.email_address)
                        contact.groups.add(group)  # signal "contact_group_signal" change "first_group" field, to use contact instace remember to reload
                        joined += 1
                    except Exception, e:
                        messages.error(
                            request,
                            "ATTENZIONE! email %s %s" % (acomea_email_obj.email_object.email_address, e)
                        )

                # PRINT MESSAGES
                if joined == 0:
                    messages.error(request, _('0 contact joined to %s' % group.name))
                if joined > 0:
                    messages.info(request, _('%(joined)s succesfully joined to %(group_name)s' % {
                        'joined': joined,
                        'group_name': group.name
                    }))

            return HttpResponseRedirect(request.get_full_path())

        if not form:
            form = JoinToGroupForm(
                initial={
                    '_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME),
                    'select_across': request.POST.get('select_across'),
                }
            )

        context = {'objects': queryset, 'form': form, }
        return render_to_response(
            'admin/contact/join_to_group.html',
            context, RequestContext(request)
        )


class CampaignAdmin(ExtendibleModelAdmin):

    prepopulated_fields = {"slug": ("title",)}

    list_display = (
        'title', 'slug', 'date_publication_start', 'date_publication_end', 'status',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'status',
                ('title', 'slug',),
                ('date_publication_start', 'date_publication_end',),
                'description',
            ),
        }),
    )
    list_filter = ('title', 'slug')
    advanced_search_fields = (
        ('title__icontains', 'slug__icontains'),
        ('status__icontains',),
        (
            'date_publication_start__gte:Data publicazione inizio',
            'date_publication_start__lt:Data publicazione inizio',
        ),
        (
            'date_publication_end__gte:Data publicazione fine',
            'date_publication_end__lt:Data publicazione fine',
        ),
    )


class SourceAdmin(ExtendibleModelAdmin):

    prepopulated_fields = {"slug": ("title",)}

    list_display = (
        'title', 'slug', 'date_publication_start', 'date_publication_end', 'status',
        'get_edit_icon', 'get_delete_icon',
    )

    fieldsets = (
        (_('logs admin'), {
            'fields': (
                'creator', 'date_create',
                'last_modifier', 'date_last_modify',
            ),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'status',
                ('title', 'slug',),
                ('date_publication_start', 'date_publication_end',),
                'description',
            ),
        }),
    )
    list_filter = ('title', 'slug')
    advanced_search_fields = (
        ('title__icontains', 'slug__icontains'),
        ('status__icontains',),
        (
            'date_publication_start__gte:Data publicazione inizio',
            'date_publication_start__lt:Data publicazione inizio',
        ),
        (
            'date_publication_end__gte:Data publicazione fine',
            'date_publication_end__lt:Data publicazione fine',
        ),
    )


admin.site.register(Contact, ContactAdmin)
admin.site.register(Role, RoleAdmin)
admin.site.register(Title, TitleAdmin)
admin.site.register(Category, CategoryAdmin)
admin.site.register(Group, GroupAdmin)
admin.site.register(Interest, InterestAdmin)
admin.site.register(Product, ProductAdmin)
admin.site.register(Network, NetworkAdmin)
admin.site.register(Email, EmailAdmin)
admin.site.register(PhoneCategory, PhoneCategoryAdmin)
admin.site.register(Phone, PhoneAdmin)
admin.site.register(CategoryProfile, CategoryProfileAdmin)
admin.site.register(WebProfile, WebProfileAdmin)
admin.site.register(AddressScope, AddressScopeAdmin)
admin.site.register(Address, AddressAdmin)
admin.site.register(AcomeaEmailObject, AcomeaEmailObjectAdmin)

admin.site.register(Campaign, CampaignAdmin)
admin.site.register(Source, SourceAdmin)
