# -*- coding: utf-8 -*-

from django import forms
from django.db import models
from django.db.models.sql.constants import QUERY_TERMS
from django.utils import six
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _

from jmb.filters import Filter, ChoiceFilter, ModelMultipleChoiceFilter, MultipleChoiceFilter
from jmb.filters.fields import LookupTypeField
from jmb.filters.filters import LOOKUP_TYPES, LOOKUP_MAP
from jmb.filters.filterset import add_suitable_choice_defaults, get_declared_filters, filters_for_model, FILTER_FOR_DBFIELD_DEFAULTS
from jmb.filters.filterset import BaseFilterSet

from django.db.models.sql.constants import LOOKUP_SEP


class FilterSetOptions(object):
    def __init__(self, options=None):
        self.model = getattr(options, 'model', None)
        self.fields = getattr(options, 'fields', None)
        self.exclude = getattr(options, 'exclude', None)

        self.order_by = getattr(options, 'order_by', False)

        self.form = getattr(options, 'form', forms.Form)


class FilterSetMetaclass(type):
    def __new__(cls, name, bases, attrs):
        try:
            parents = [b for b in bases if issubclass(b, FilterSet)]
        except NameError:
            # We are defining FilterSet itself here
            parents = None
        new_class = super(
            FilterSetMetaclass, cls).__new__(cls, name, bases, attrs)
        opts = new_class._meta = FilterSetOptions(
            getattr(new_class, 'Meta', None))
        declared_filters = get_declared_filters(bases, attrs, model=opts.model)

        if not parents:
            return new_class

        # fields is a list of possibly 1-level nested fields
        field_names = []
        for item in opts.fields:
             field_names += [item] if isinstance(item, basestring) else item
        if opts.model:
            filters = filters_for_model(opts.model, field_names, opts.exclude,
                                        new_class.filter_for_field,
                                        new_class.filter_for_reverse_field)
            filters.update(declared_filters)
        else:
            filters = declared_filters

        if None in filters.values():
            raise TypeError("Meta.fields contains a field that isn't defined "
                "on this FilterSet %s" % [key for key, val in filters.iteritems() if not val])

        new_class.declared_filters = declared_filters
        new_class.base_filters = filters
        return new_class


LOOKUP_MAP['isnull'] = _('is NULL'),


class IsNullFilter(Filter):
    field_class = forms.NullBooleanField

    def filter(self, qs, value):
        if isinstance(value, bool):
            query_key = '{0}__isnull'.format(self.name)
            return qs.filter(**{query_key: value})
        return qs

    @property
    def field(self):
        if not hasattr(self, '_field'):
            if (self.lookup_type is None or
                    isinstance(self.lookup_type, (list, tuple))):
                if self.lookup_type is None:
                    lookup = [(x, LOOKUP_MAP.get(x, x)) for x in LOOKUP_TYPES]
                else:
                    lookup = [
                        (x, LOOKUP_MAP.get(x, x)) for x in self.lookup_type if x in LOOKUP_TYPES]
                self._field = LookupTypeField(self.field_class(
                    required=self.required, widget=self.widget, **self.extra),
                    lookup, required=self.required, label=self.label)
            else:
                self._field = self.field_class(required=self.required,
                    label=self.label, widget=self.widget, **self.extra)
        return self._field


FILTER_FOR_DBFIELD_DEFAULTS['IsNull'] = {
        'filter_class': IsNullFilter
    }


class FilterSet(six.with_metaclass(FilterSetMetaclass, BaseFilterSet)):

	@classmethod
	def filter_for_field(cls, field, name, model, label):
		"""Mapping of field_name to filter

		:arg field:
		:arg name: the name of the field (a key of queryset filter)
		:arg model: the model of the
		:arg label: the label to be used
		"""
		parts = name.split(LOOKUP_SEP)
		if parts[-1] in QUERY_TERMS or parts[-1] == 'year':
			name = LOOKUP_SEP.join(parts[:-1])
			lookup_type = parts[-1]
		else:
			lookup_type = 'exact'

		filter_for_field = dict(FILTER_FOR_DBFIELD_DEFAULTS)
		filter_for_field.update(cls.filter_overrides)

		label = capfirst(_(label or field.name)) + (
			'' if lookup_type in ('exact', 'in') else " (%s)" % LOOKUP_MAP.get(lookup_type, ''))
		default = {
			'name': name,
			'label': label,
			'lookup_type': lookup_type,
		}
		if field.choices:
			default['choices'] = field.choices
			if lookup_type == 'in':
				default['lookup_type'] = ('in_any', 'not_in_any')
				return MultipleChoiceFilter(**default)
			else:
				return ChoiceFilter(**default)

		if lookup_type == 'range':
			data = filter_for_field.get('DateRange')  # implements __range
		elif lookup_type == 'isnull':
			data = filter_for_field.get('IsNull')
		else:
			data = filter_for_field.get(field.__class__)
		if data is None:
			# could be a derived field, inspect parents
			for class_ in field.__class__.mro():
				# skip if class_ is models.Field or object
				# 1st item in mro() is original class
				if class_ in (field.__class__, models.Field, object):
					continue
				data = filter_for_field.get(class_)
				if data:
					break
			if data is None:
				return
		if lookup_type == 'in':
			# 'Non model' cases are already considered
			filter_class = ModelMultipleChoiceFilter
			if isinstance(field, models.ForeignKey):
				default['lookup_type'] = ('in_any', 'not_in_any')
			else:
				default['lookup_type'] = ('in_any', 'not_in_any', 'in_all', 'not_in_all')
		else:
			filter_class = data.get('filter_class')
		default.update(data.get('extra', lambda field: {})(field))
		if filter_class is not None:
			filter_ = filter_class(**default)
			add_suitable_choice_defaults(filter_, model, name)
			return filter_
