# -*- coding: utf-8 -*-
import datetime
import logging
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.core.validators import validate_email
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template
from django.template import Context
from django.conf import settings
from django.contrib.auth.models import User
from jmb.core.forms import DateTimePicker

from jmb.newsletter.models import Newsletter
from jmb.newsletter2.models import Newsletter as Newsletter2
from acomea.contact.models import Contact, Group, Email, Phone, PhoneCategory
from acomea.contact.utils.generic import send_contact_confirm_email

from acomea.profilo.models import AcomeaProfilo, RichiestaAttivazioneCliente
from acomea.event.models import Event

log = logging.getLogger('acomea.contact')


class SetNewsletterForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    newsletter = forms.ModelChoiceField(Newsletter.objects)


class SetNewsletter2Form(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    newsletter = forms.ModelChoiceField(Newsletter2.objects)


class SetInviteForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    event = forms.ModelChoiceField(Event.objects)
    send_from_date = forms.DateTimeField(required=False, widget=DateTimePicker)
    send_to_date = forms.DateTimeField(required=False, widget=DateTimePicker)

    def clean(self):
        cleaned_data = super(SetInviteForm, self).clean()
        send_from = cleaned_data.get('send_from_date', False)
        send_to = cleaned_data.get('send_to_date', False)
        if bool(send_from) != bool(send_to):
            raise forms.ValidationError(
                _(u"I campi data devono essere entrambi valorizzati o nulli")
            )
        if send_from and send_to:
            if send_from >= send_to:
                raise forms.ValidationError(
                    _(u"La data di inizio deve essere successiva a quella di fine")
                )
            if send_from <= datetime.datetime.now():
                raise forms.ValidationError(
                    _(u"La data di inizio non può essere precedente a quella attuale")
                )
        return cleaned_data


# Attualmente inutilizzato
class SetInviteNoControlsForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    event = forms.ModelChoiceField(Event.objects)


class SetBonificaForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)


class JoinToGroupForm(forms.Form):
    _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
    select_across = forms.IntegerField(widget=forms.HiddenInput)
    group = forms.ModelChoiceField(Group.objects)


class SetNewsletterExtraForm(forms.Form):
    newsletter = forms.ModelChoiceField(
        Newsletter.objects,
        label="newsletter"
    )
    emails = forms.CharField(
        label="emails",
        widget=forms.Textarea,
        help_text=_("insert emails with  separated by ';'")
    )

    def clean_emails(self):
        emails = self.cleaned_data['emails'].split(';')
        list_emails = []
        for email in emails:
            email = email.strip().lower()
            validate_email(email)
            list_emails.append(email)
        return ";".join(list_emails)


class SetNewsletter2ExtraForm(forms.Form):
    newsletter = forms.ModelChoiceField(
        Newsletter2.objects,
        label="newsletter"
    )
    emails = forms.CharField(
        label="emails",
        widget=forms.Textarea,
        help_text=_("insert emails with  separated by ';'")
    )

    def clean_emails(self):
        emails = self.cleaned_data['emails'].split(';')
        list_emails = []
        for email in emails:
            email = email.strip().lower()
            validate_email(email)
            list_emails.append(email)
        return ";".join(list_emails)


class NewsletterSubscriptionForm(forms.ModelForm):
    class Meta:
        model = Contact
        fields = (
            'first_name', 'last_name', 'organization',
            'auth_trattamento_dati', 'auth_materiale_commerciale',
            'auth_societa_terze'
        )

    def __init__(self, *args, **kwargs):
        active_groups = Group.objects.filter(id__in=(1, 2, 3), status=1)
        super(NewsletterSubscriptionForm, self).__init__(*args, **kwargs)
        self.fields['first_name'].widget.attrs = {
            'class': 'span8',
            'placeholder': _("Last name")
        }
        self.fields['last_name'].widget.attrs = {
            'class': 'span8',
            'placeholder': _("First name")
        }
        self.fields['organization'].widget.attrs = {
            'class': 'span8',
            'placeholder': _("Organization")
        }

        self.fields['email'] = forms.EmailField(
            widget=forms.TextInput(
                attrs={
                    'class': 'span8',
                    'placeholder': _("Email")
                }
            )
        )
        self.fields['mobile'] = forms.CharField(
            required=False, widget=forms.TextInput(
                attrs={
                    'class': 'span8',
                    'placeholder': _("Mobile")
                }
            )
        )
        self.fields['locality'] = forms.CharField(
            required=False, widget=forms.TextInput(
                attrs={'class': 'span8',  'placeholder': _("locality")}
            )
        )
        self.fields['professionale'] = forms.BooleanField(
            required=False,
            label=u"Dichiaro di essere un investitore professionale (es. Consulente finanziario, Private Banker...)",
        )
        self.fields['auth_trattamento_dati'].label = settings.AUTH_TRATTAMENTO_DATI_LABEL
        self.fields['auth_trattamento_dati'].widget = forms.RadioSelect(
            choices=settings.AUTH_CHOICES,
        )
        self.fields['auth_materiale_commerciale'].label = settings.AUTH_MATERIALE_COMMERCIALE_LABEL
        self.fields['auth_materiale_commerciale'].widget = forms.RadioSelect(
            choices=settings.AUTH_CHOICES
        )
        # self.fields['auth_societa_terze'].label = settings.AUTH_SOCIETA_TERZE_LABEL
        # self.fields['auth_societa_terze'].widget = forms.RadioSelect(
        #     choices=settings.AUTH_CHOICES
        # )

        # fixme: traduzione labels forzate qui (da spostare)
        self.fields['first_name'].label = "Nome"
        self.fields['last_name'].label = "Cognome"
        self.fields['mobile'].label = "Cellulare"
        self.fields['organization'].label = "Organizzazione"
        self.fields['locality'].label = "Localita'"

        self.fields.keyOrder = [
            'first_name', 'last_name', 'organization',
            'email', 'mobile', 'locality', 'professionale',
            'auth_trattamento_dati', 'auth_materiale_commerciale']
        self.auth_fields = ['auth_trattamento_dati', 'auth_materiale_commerciale']
        for key in self.auth_fields:
            self.fields[key].initial = None
            self.fields[key].widget.attrs = {'class': 'pull-right'}

    def clean_email(self):
        email = self.cleaned_data['email']
        if Email.objects.filter(email=email).count():
            raise forms.ValidationError("Email already exists.")
        return email

    def clean_auth_trattamento_dati(self):
        auth_trattamento_dati = self.cleaned_data.get('auth_trattamento_dati')
        if auth_trattamento_dati:
            return auth_trattamento_dati
        raise forms.ValidationError(settings.AUTH_TRATTAMENTO_DATI_ERR_MSG)

    def clean_auth_materiale_commerciale(self):
        auth_materiale_commerciale = self.cleaned_data.get('auth_materiale_commerciale')
        if auth_materiale_commerciale:
            return auth_materiale_commerciale
        raise forms.ValidationError(settings.AUTH_MATERIALE_COMMERCIALE_ERR_MSG)

    def clean_auth_societa_terze(self):
        auth_societa_terze = self.cleaned_data.get('auth_societa_terze')
        return auth_societa_terze

    def save(self):
        data = self.cleaned_data

        # salvo contatto
        contact = super(NewsletterSubscriptionForm, self).save()
        try:
            if 'professionale' in data and data['professionale']:
                investitori_professionali_group = Group.objects.get(
                    name=settings.CONTACT_GROUP_INVESTITORI_PROFESSIONALI
                )
                contact.groups.add(investitori_professionali_group)
            else:
                newsletter_privati_group = Group.objects.get(
                    name=settings.CONTACT_GROUP_NEWSLETTER_PRIVATI
                )
                contact.groups.add(newsletter_privati_group)
            contact = contact.get_reload_from_db()  # to consider signal "contact_group_signal" manipulation
        except Group.DoesNotExist:
            pass

        if data['locality']:
            contact.note = data['locality']
            contact.save()

        # salvo data iscrizione
        contact.date_subscription = datetime.datetime.now()
        contact.save()
        # set_bonifica(contact) -> bonifica settata alla conferma contatto (ConfirmContactPlugin)
        # associo gruppo
        #contact.groups.add(data['groups'])

        # salvo email
        email = Email(email=data['email'], default=True)
        contact.emails.add(email)

        # salvo mobile
        if data['mobile']:
            phone_category = PhoneCategory.objects.get(pk=1)
            phone = Phone(phone=data['mobile'], category=phone_category)
            contact.phones.add(phone)

    def send(self, instance, request=None):
        data = self.cleaned_data
        if request:
            send_contact_confirm_email(
                contact=self.instance, email=data.get('email'), request=request
            )
        else:
            html_template = get_template(instance.email_template_name)
            subject = 'Iscrizione alla newsletter'
            from_email = instance.sent_from
            to = data['email']
            data_dict = Context({'data': data})
            html_content = html_template.render(data_dict)
            msg = EmailMultiAlternatives(
                subject,
                html_content,
                from_email,
                [to]
            )
            msg.content_subtype = "html"
            msg.send()


class TraceRequestsForm(forms.ModelForm):
    """
    Base Trace request form without auth fields
    """
    class Meta:
        model = Contact
        fields = (
            'first_name', 'last_name',
        )

    def __init__(self, *args, **kwargs):
        self.mandatory_amc = kwargs.pop('mandatory_amc', False)
        self.mandatory_amc_err_message = kwargs.pop('mandatory_amc_err_message', "")
        super(TraceRequestsForm, self).__init__(*args, **kwargs)
        self.fields['first_name'].widget.attrs = {
            'class': 'form-control required',
            'placeholder': 'NOME'
        }
        self.fields['last_name'].widget.attrs = {
            'class': 'form-control required',
            'placeholder': 'COGNOME'
        }
        self.fields['email'] = forms.EmailField(
            widget=forms.TextInput(
                attrs={
                    'class': 'form-control required',
                    'placeholder': 'EMAIL'
                }
            )
        )
        self.fields['phone'] = forms.CharField(
            required=False, widget=forms.TextInput(
                attrs={
                    'class': 'form-control required',
                    'placeholder': 'TELEFONO(facoltativo)'
                }
            )
        )

    def save(self, commit=True):
        self.cleaned_data['email'] = self.cleaned_data['email'].lower()
        email = self.cleaned_data['email']
        contacts = Contact.objects.filter(emails__email=email)
        created = True

        if contacts:
            contact = contacts[0]
            created = False
            if commit:
                contact.save()
        else:
            contact = super(TraceRequestsForm, self).save(commit=False)
            contact.bonifica = False
            if commit:
                contact.save()
                email = Email(
                    email=email,
                    default=True,
                    contact=contact
                )
                email.save()
        if commit:
            contact.save()

        return contact, created


class PresetTraceRequestsForm(TraceRequestsForm):
    """Versione della form che forza auth_trattamento_dati = True
    """

    def __init__(self, *args, **kwargs):
        super(PresetTraceRequestsForm, self).__init__(*args, **kwargs)
        

class ExtendedTraceRequestsForm(TraceRequestsForm):
    """
    Trace request form with auth fields
    """
    class Meta:
        model = Contact
        fields = (
            'first_name', 'last_name',
            'auth_trattamento_dati', 'auth_materiale_commerciale', 'auth_societa_terze'
        )

    def __init__(self, *args, **kwargs):
        super(ExtendedTraceRequestsForm, self).__init__(*args, **kwargs)
        self.fields.keyOrder = [
            'first_name', 'last_name',
            'email', 'phone',
            'auth_trattamento_dati', 
            'auth_materiale_commerciale'
        ]
        self.auth_fields = ['auth_trattamento_dati', 'auth_materiale_commerciale']
        for key in self.auth_fields:
            self.fields[key] = forms.ChoiceField(
                choices=settings.AUTH_CHOICES,
                required=False,
                widget=forms.RadioSelect(),
            )
            self.fields[key].widget.attrs = {'class': 'pull-right'}
            #self.fields[key].widget.attrs = {'disabled':'disabled'}
        self.fields['auth_materiale_commerciale'].label = settings.AUTH_MATERIALE_COMMERCIALE_LABEL
        self.fields['auth_trattamento_dati'].required = True
        self.fields['auth_trattamento_dati'].initial = True
        self.fields['auth_trattamento_dati'].label = settings.AUTH_TRATTAMENTO_DATI_LABEL
        self.fields['auth_societa_terze'].label = settings.AUTH_SOCIETA_TERZE_LABEL

    def clean_auth_trattamento_dati(self):
        auth_trattamento_dati = self.cleaned_data.get('auth_trattamento_dati', 'False')
        if auth_trattamento_dati == "True":
            return auth_trattamento_dati
        raise forms.ValidationError(settings.AUTH_TRATTAMENTO_DATI_ERR_MSG)

    def clean_auth_materiale_commerciale(self):
        auth_materiale_commerciale = self.cleaned_data.get('auth_materiale_commerciale', False)
        if auth_materiale_commerciale:
            if self.mandatory_amc:
                if not auth_materiale_commerciale == "True":
                    if self.mandatory_amc_err_message:
                        err_msg = unicode(self.mandatory_amc_err_message)
                    else:
                        err_msg = settings.AUTH_MATERIALE_COMMERCIALE_ERR_MSG
                    raise forms.ValidationError(
                        err_msg
                    )
            return auth_materiale_commerciale
        raise forms.ValidationError(settings.AUTH_MATERIALE_COMMERCIALE_ERR_MSG)

    def clean_auth_societa_terze(self):
        auth_societa_terze = self.cleaned_data.get('auth_societa_terze')
        return auth_societa_terze


class PresetExtendedTraceRequestsForm(ExtendedTraceRequestsForm):

    def __init__(self, *args, **kwargs):
        super(PresetExtendedTraceRequestsForm, self).__init__(*args, **kwargs)
        del self.fields["auth_trattamento_dati"]


    def save(self, *args, **kwargs):
        contact, created = super(PresetExtendedTraceRequestsForm, self).save(*args, **kwargs)
        contact.auth_trattamento_dati = True
        contact.save()
        return contact, created


class NewsletterUnsubscriptionForm(forms.Form):
    email = forms.EmailField()
    email.widget.attrs = {'class': 'span8'}

    def send(self, instance):
        data = self.cleaned_data
        emails = Email.objects.filter(
            email=data['email'],
            status=1,
            contact__date_unsubscription__isnull=True,
            contact__status=1
        )
        if emails:
            html_template = get_template(instance.email_template_name)
            subject, from_email, to = 'Disiscrizione dalla newsletter', instance.sent_from, data['email']
            data_dict = Context({'data': data})
            html_content = html_template.render(data_dict)
            msg = EmailMultiAlternatives(subject, html_content, from_email, [to])
            msg.content_subtype = "html"
            msg.send()


class PrivacyManagementForm(forms.ModelForm):
    class Meta:
        model = Contact
        fields = (
            'auth_trattamento_dati', 'auth_materiale_commerciale'
        )

    def __init__(self, *args, **kwargs):

        profilo = kwargs.pop('profilo', None)
        if profilo:
            self.profilo = profilo
        super(PrivacyManagementForm, self).__init__(*args, **kwargs)
        self.auth_fields = ['auth_trattamento_dati', 'auth_materiale_commerciale']

        self.fields['first_name'] = forms.CharField(
            max_length=30,
            required=False,
            widget=forms.TextInput(attrs={'placeholder': 'Nome'})
        )

        self.fields['last_name'] = forms.CharField(
            max_length=30,
            required=False,
            widget=forms.TextInput(attrs={'placeholder': 'Cognome'})
        )
        self.fields['indirizzo'] = forms.CharField(
            max_length=128,
            required=False,
            widget=forms.TextInput(attrs={'placeholder': 'Indirizzo'})
        )
        self.fields['citta'] = forms.CharField(
            max_length=64,
            required=False,
            widget=forms.TextInput(attrs={'placeholder': 'Città'})
        )
        self.fields['provincia'] = forms.CharField(
            max_length=2,
            required=False,
            widget=forms.TextInput(attrs={'placeholder': 'Provincia'})
        )
        self.fields['cap'] = forms.CharField(
            max_length=5,
            required=False,
            widget=forms.TextInput(attrs={'placeholder': 'CAP'})
        )
        self.fields['stato'] = forms.CharField(
            max_length=30,
            required=False,
            widget=forms.TextInput(attrs={'placeholder': 'Paese'})
        )

        for key in self.auth_fields:
            self.fields[key].widget = forms.RadioSelect(
                choices=settings.AUTH_CHOICES,
            )
            self.fields[key].widget.attrs = {'class': 'pull-right'}
            self.fields[key].widget.attrs = {'disabled': 'disabled'}

        self.user_fields = ['first_name', 'last_name']
        self.rac_fields = ['indirizzo', 'citta', 'provincia', 'cap', 'stato']

        for key in self.user_fields+self.rac_fields:
            self.fields[key].widget.attrs['class'] = "span10 offset1"

        if profilo:
            try:
                self.fields['first_name'].initial = profilo.user.first_name
                self.fields['last_name'].initial = profilo.user.last_name
                self.fields['indirizzo'].initial = profilo.rac.indirizzo_spedizione
                self.fields['citta'].initial = profilo.rac.citta_spedizione
                self.fields['provincia'].initial = profilo.rac.provincia_spedizione
                self.fields['cap'].initial = profilo.rac.cap_spedizione
                self.fields['stato'].initial = profilo.rac.stato_spedizione

                if not profilo.is_only_lead:
                    for key in self.user_fields+self.rac_fields:
                        if key in self.fields:
                            self.fields[key].widget.attrs['disabled']='disabled'

            except User.DoesNotExist:
                pass
            except RichiestaAttivazioneCliente.DoesNotExist:
                pass

            self.fields['auth_trattamento_dati'].label = settings.AUTH_TRATTAMENTO_DATI_LABEL
            self.fields['auth_materiale_commerciale'].label = settings.AUTH_MATERIALE_COMMERCIALE_LABEL

            for field in self.auth_fields:
                self.fields[field].widget.attrs = {'class': 'pull-right'}

            if profilo:
                if profilo.is_cliente:
                    self.fields['auth_trattamento_dati'].widget.attrs['disabled'] = 'disabled'
            self.fields.keyOrder = [
                'first_name', 'last_name',
                'indirizzo', 'citta', 'provincia', 'cap', 'stato',
                'auth_trattamento_dati',
                'auth_materiale_commerciale']

    def clean_auth_trattamento_dati(self):
        auth_trattamento_dati = self.cleaned_data.get('auth_trattamento_dati', None)
        if self.profilo:
            if not auth_trattamento_dati and self.profilo.is_cliente:
                auth_trattamento_dati = True
        return auth_trattamento_dati
    
    def save(self, commit=True):
        contact = super(PrivacyManagementForm, self).save(commit)
        contact.bonifica = True
        contact.save()

        iscrizione_alla_newsletter = self.cleaned_data.get('iscrizione_alla_newsletter', None)
        try:
            profilo = self.instance.user.profilo
        except AttributeError, e:
            profilo = None
        except AcomeaProfilo.DoesNotExist:
            profilo = None
        if profilo:
            #profilo.iscrizione_alla_newsletter = iscrizione_alla_newsletter
            #profilo.save()
            self._update_user_rac_contact_data(profilo)

    def _update_user_rac_contact_data(self, profilo=None):
        """
        aggiorna parametri dello user e del profilo con quelli passati
        """
        data = self.cleaned_data
        try:
            user = profilo.user
        except User.DoesNotExist:
            user = None

        # user = kwargs.pop("user", None)
        # profilo = kwargs.pop("profilo", None)
        if profilo and user:
            if profilo.is_only_lead:
                try:
                    if 'first_name' in data:
                        profilo.user.first_name = data['first_name']
                    if 'last_name' in data:
                        profilo.user.last_name = data['last_name']
                    if 'indirizzo' in data:
                        profilo.rac.indirizzo_spedizione = data['indirizzo']
                    if 'citta' in data:
                        profilo.rac.citta_spedizione = data['citta']
                    if 'provincia' in data:
                        profilo.rac.provincia_spedizione = data['provincia']
                    if 'cap' in data:
                        profilo.rac.cap_spedizione = data['cap']
                    if 'stato' in data:
                        profilo.rac.stato_spedizione = data['stato']
                except (User.DoesNotExist, AcomeaProfilo.DoesNotExist):
                    pass
                profilo.user.save()
                profilo.rac.save()
                try:
                    user.contact.first_name = user.first_name
                    user.contact.last_name = user.last_name
                    user.contact.save()
                except Contact.DoesNotExist:
                    pass


class AuthPrivacyManagementForm(forms.ModelForm):
    class Meta:
        model = Contact
        fields = ('auth_materiale_commerciale',)

    def __init__(self, *args, **kwargs):
        profilo = kwargs.pop('profilo', None)
        super(AuthPrivacyManagementForm, self).__init__(*args, **kwargs)

        self.fields['auth_materiale_commerciale'].required = True
        self.fields['auth_materiale_commerciale'].widget = forms.RadioSelect(
            choices=settings.AUTH_CHOICES,
        )
        self.fields['auth_materiale_commerciale'].widget.attrs = {'class': 'pull-right'}
        self.fields['auth_materiale_commerciale'].label = settings.AUTH_MATERIALE_COMMERCIALE_LABEL

    def clean_auth_materiale_commerciale(self):
        auth_materiale_commerciale = self.cleaned_data['auth_materiale_commerciale']
        if auth_materiale_commerciale is None:
            raise forms.ValidationError('Prego effettuare una scelta')
        return auth_materiale_commerciale

    def save(self, commit=True):
        contact = super(AuthPrivacyManagementForm, self).save(commit)
        contact.bonifica = True
        contact.auth_trattamento_dati = True
        contact.save()


class ConfirmPrivacyForm(forms.ModelForm):
    """
    privacy confirm form class
    """
    class Meta:
        model = Contact
        fields = (
            'auth_materiale_commerciale',
        )

    def __init__(self, *args, **kwargs):
        email_param = kwargs.pop('email', None)
        contact_id_param = kwargs.pop('contact_id', None)
        super(ConfirmPrivacyForm, self).__init__(*args, **kwargs)
        if email_param:
            self.initialize_disabled_fields(email=email_param)
        if contact_id_param:
            self.initialize_disabled_fields(contact_id=contact_id_param)

    def clean_auth_trattamento_dati(self):
        if not 'auth_trattamento_dati' in self.data:
            self.cleaned_data.pop('auth_trattamento_dati')
            return None
        auth_trattamento_dati = self.cleaned_data.get('auth_trattamento_dati', None)
        return auth_trattamento_dati

    def clean_auth_materiale_commerciale(self):
        if not 'auth_materiale_commerciale' in self.data:
            auth_materiale_commerciale = None
        else:
            auth_materiale_commerciale = self.cleaned_data.get('auth_materiale_commerciale')
        if not auth_materiale_commerciale is None:
            return auth_materiale_commerciale
        raise forms.ValidationError(settings.AUTH_MATERIALE_COMMERCIALE_ERR_MSG)

    def initialize_disabled_fields(self, email=None, contact_id=None):
        """
        """
        if not email and not contact_id:
            return None
        self.auth_fields = ['auth_trattamento_dati', 'auth_materiale_commerciale']
        self.fields['email'] = forms.EmailField(
            required=False,
            max_length=150,
            widget=forms.TextInput(attrs={'placeholder': 'Email'})
        )
        self.fields['first_name'] = forms.CharField(
            max_length=150,
            required=False,
            widget=forms.TextInput(attrs={'placeholder': 'Nome'})
        )
        self.fields['last_name'] = forms.CharField(
            max_length=150,
            required=False,
            widget=forms.TextInput(attrs={'placeholder': 'Cognome'})
        )
        self.fields['auth_trattamento_dati'] = forms.BooleanField(
            required=False,
        )
        if email and not contact_id:
            try:
                email = Email.objects.get(email=email)
                self.instance = email.contact
            except (Email.DoesNotExist, Email.MultipleObjectsReturned, Contact.DoesNotExist):
                return None

        if contact_id and not email:
            try:
                contact = Contact.objects.get(pk=contact_id)
                self.instance = contact
            except (Contact.DoesNotExist, Contact.MultipleObjectsReturned):
                return None
        try:
            self.fields['email'].initial = self.instance.emails.filter(default=True)[0]
        except IndexError:
            self.fields['email'].initial = self.instance.emails.all()[0]
        self.fields['first_name'].initial = self.instance.first_name
        self.fields['last_name'].initial = self.instance.last_name
        self.fields['auth_trattamento_dati'].initial = True
        self.fields['auth_materiale_commerciale'].initial = None
        if self.instance.bonifica:
            self.fields['auth_materiale_commerciale'].initial = self.instance.auth_materiale_commerciale
        for key in self.auth_fields:
            self.fields[key].widget = forms.RadioSelect(
                choices=settings.AUTH_CHOICES,
            )
            self.fields[key].widget.attrs = {'class': 'pull-right'}
            self.fields[key].widget.attrs = {'disabled': 'disabled'}
        self.fields.keyOrder = [
            'first_name', 'last_name', 'email',
            'auth_trattamento_dati', 'auth_materiale_commerciale']
        for key in ('first_name', 'last_name', 'email'):
            self.fields[key].widget.attrs['class'] = "span9 offset1"
        for field in self.auth_fields:
                self.fields[field].widget.attrs = {'class': 'pull-left'}
        for field_name in self.fields.keyOrder[:-1]:
            self.fields[field_name].widget.attrs['disabled'] = 'disabled'
            self.fields[field_name].required = False
        self.fields['auth_trattamento_dati'].label = settings.AUTH_TRATTAMENTO_DATI_LABEL
        self.fields['auth_materiale_commerciale'].label = settings.AUTH_MATERIALE_COMMERCIALE_LABEL

    def save(self, commit=True):
        contact = self.instance
        if contact.pk is None:
            pk = self.data.get('contact_id', None)
            if pk is None:
                return None
            try:
                contact = Contact.objects.get(pk=pk)
            except (Contact.DoesNotExist, Contact.MultipleObjectsReturned):
                return None
        contact.auth_materiale_commerciale = self.cleaned_data.get('auth_materiale_commerciale')
        contact.save()
        return contact
