# coding: utf-8
"""
.. _sync_contact_group:

Sync Contact Group
==================

Command to align "ContactGroup" model on through between "Contact" and "Group".
Note ContactGroup is a model to store "date_create" and "date_last_modify"

usage::

  dj sync_contact_group [--dry-run]

"""


import ast
import logging
from optparse import make_option

from django.core.management.base import BaseCommand
from django.db import connections

from acomea.contact import models

# LOGGER
logger = logging.getLogger('acomea.contact')


QUERY_RETRIEVE_UNALIGNED = """
SELECT DISTINCT(T1.contact_id, T1.group_id)
FROM contact_contact_groups AS T1 LEFT JOIN public.contact_contactgroup AS T2
    ON T1.group_id = T2.group_id AND T1.contact_id = T2.contact_id
WHERE T2.id IS NULL;
"""


class Command(BaseCommand):

    option_list = BaseCommand.option_list + (
        # debug level in console
        make_option(
            '-l', '--debug-level', default='debug',
            help='Set log level (debug, info, warnings) for file.'
                 '"tail -24lf /var/log/django/www.acomea.it.contact.log" to see',
        ),
        make_option(
            '--dry-run', action='store_true', dest='dry_run', default=False,
            help="Don't sync mailup, just the local db",
        )
    )

    def handle(self, *args, **options):

        log_level = getattr(logging,  options.get('debug_level', 'info').upper())
        logger.setLevel(log_level)

        dry_run = options.get('dry_run')

        cursor = connections['default'].cursor()
        cursor.execute(QUERY_RETRIEVE_UNALIGNED)
        result = cursor.fetchall()
        logger.info('Found {} row to insert in ContactGroup model'.format(str(len(result))))

        # report vars
        num_created = 0
        num_skipped = 0
        group_names = []

        for line in result:
            try:
                list_ids = ast.literal_eval(line[0])
                contact_id = list_ids[0]
                group_id = list_ids[1]
                if not dry_run:
                    obj, created = models.ContactGroup.objects.get_or_create(
                        contact_id=contact_id, group_id=group_id
                    )
                    if created:
                        logger.debug("{} created".format(list_ids))
                        num_created += 1
                    else:
                        logger.debug("{} skipped".format(list_ids))
                        num_skipped += 1

                    # group name report
                    group_names.append(obj.group.name)
                else:
                    logger.info('''DRY RUN: skipped query:
obj, created = models.ContactGroup.objects.get_or_create(
    contact_id={contact_id}, group_id={group_id}
)
'''.format(contact_id=contact_id, group_id=group_id))
            except Exception as e:
                logger.critical(e)

        logger.info('created: {}, skipped: {}'.format(num_created, num_skipped))
        logger.info('names:{}'.format(str(list(set(group_names)))))
