# -*- coding: utf-8 -*-
import datetime
import logging

from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.signals import m2m_changed, post_delete

from django.conf import settings
from django.contrib.auth.models import User
from django.utils.timezone import utc
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel, StatusModel, DefaultModel, SEOModel, OrderedModel
from jmb.core.middleware.thread_local import get_current_user, get_request
from jmb.core.utils.functions import slugify
from jmb.l10n.models import Country
from jmb.mailer.models import EmailObject

from acomea.contact.signals import contact_group_signal, delete_mailup_recipient

from . import managers

privacy_log = logging.getLogger('acomea.privacy')


def register_contact(nome, cognome, email, gruppi, societa=None, cellulare=None, citta=None):
    # stringa, stringa, stringa, qs Group.objects, stringa, stringa, stringa
    # se esiste la mail la mette a default e aggiunge eventuali gruppi non ancora aggiunti
    # altrimenti crea il contatto e gli aggiunge i gruppi
    create = False
    if Email.objects.filter(email__exact=email).count() > 0:
        mail = Email.objects.get(email=email)
        mail.default = True
        mail.save()
        contact = mail.contact
    else:
        contact = Contact(first_name=nome, last_name=cognome)
        contact.save()
        email = Email(contact=contact, email=email, default=True)
        email.save()

        create = True
    for g in gruppi:
        contact.groups.add(g)
        contact = contact.get_reload_from_db()  # to consider signal "contact_group_signal" manipulation

    if create:
        if societa:
            contact.organization = societa
        contact.save()
        if cellulare:
            pc, _ = PhoneCategory.objects.get_or_create(name='cellulare')
            Phone.objects.create(contact=contact, phone=cellulare, category=pc)
        if citta:
            #asc, _ = AddressScope.objects.get_or_create(name='default')
            a = Address.objects.create(contact=contact, name='', number='', locality=citta)
    return contact


class Role(DateModel, UserModel, StatusModel):
    """
    Stores contact roles
    EXAMPLE: 'sales manager', 'development manager'
    """
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )
    show_in_registration = models.BooleanField(
        default=True,
        verbose_name=_('Show in registration'),
        help_text=_('Show in registration as option'),
    )

    class Meta:
        verbose_name = _("Contact role")
        verbose_name_plural = _("Contact roles")
        ordering = ('name',)
        permissions = (
            ('can_list_role', 'Can list Role'),
        )

    def __unicode__(self):
        return u"%s" % self.name

    def is_show_in_registration(self):
        value = 'yes' if self.show_in_registration else 'no'
        return '<img src="/static/admin/img/icon-{}.gif" alt="{}" style="margin-top:4px">'.format(value, value)
    is_show_in_registration.allow_tags = True
    is_show_in_registration.short_description = _("Show in registration")


class Title(StatusModel, OrderedModel, DateModel, UserModel):
    """
    Stores contact titles
    EXAMPLE: 'mr. mister', 'dr. doctor'
    """
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )
    abbreviation = models.CharField(
        unique=True,
        max_length=10,
        verbose_name=_("Abbreviation"),
    )

    class Meta:
        verbose_name = _("Contact title")
        verbose_name_plural = _("Contact titles")
        ordering = ('name',)
        permissions = (
            ('can_list_title', 'Can list Title'),
        )

    def __unicode__(self):
        return u"%s" % self.abbreviation


class Group(StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )
    mailup_group_id = models.IntegerField(
        null=True, blank=True,
        verbose_name=_("mailup group id")
    )

    class Meta:
        verbose_name = _("Contact group")
        verbose_name_plural = _("Contact groups")
        ordering = ('name',)
        permissions = (
            ('can_list_group', 'Can list Group'),
        )

    def __unicode__(self):
        return u"%s" % self.name


class Category(StatusModel, OrderedModel, DateModel, UserModel):
    """
    Example: cliente diretto, collocatore, prospect, istituzionale, PF, consulente
    """
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )

    class Meta:
        verbose_name = _("Contact category")
        verbose_name_plural = _("Contact categories")
        ordering = ('name',)
        permissions = (
            ('can_list_category', 'Can list Category'),
        )

    def __unicode__(self):
        return u"%s" % self.name


class Interest(StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )

    class Meta:
        verbose_name = _("Contact interest")
        verbose_name_plural = _("Contact interests")
        ordering = ('name',)
        permissions = (
            ('can_list_interest', 'Can list Interest'),
        )

    def __unicode__(self):
        return u"%s" % self.name


class Product(StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )

    class Meta:
        verbose_name = _("Contact product")
        verbose_name_plural = _("Contact products")
        ordering = ('name',)
        permissions = (
            ('can_list_product', 'Can list Product'),
        )

    def __unicode__(self):
        return u"%s" % self.name


class Network(StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )

    class Meta:
        verbose_name = _("Contact network")
        verbose_name_plural = _("Contact networks")
        ordering = ('name',)
        permissions = (
            ('can_list_network', 'Can list Network'),
        )

    def __unicode__(self):
        return u"%s" % self.name


class ContactGroup(DateModel):
    """
    Questa classe potrebbe essere la through tra Contact vs Groups, ma non è stata impostata così
    per utilizzare nell'admin il filter_horizontal
    Questa classe viene modificata dal segnale inviato dalla relazione m2m tra contact e group

    """
    contact = models.ForeignKey("Contact")
    group = models.ForeignKey("Group")

    class Meta:
        verbose_name = _("Contact groups")


class Contact(StatusModel, OrderedModel, DateModel, UserModel):

    @classmethod
    def bonifica_first_group(cls):
        contact_log = logging.getLogger('acomea.contact')
        contact_log.info(u'Contatti totali: {}'.format(cls.objects.count()))

        for i, contact in enumerate(cls.objects.all()):
            verified, first_group, first_contact_group = contact.get_verified_first_group()
            if not verified:
                contact_log.info(
                    u';{};{};{};{};{};{}'.format(
                        i, contact.id, contact.first_name, contact.last_name,
                        first_group, first_contact_group))
                contact.first_group = first_contact_group
                contact.save()

    # user è definito in fondo con una OneToOne
    # user = models.ForeignKey('auth.User', blank=True, null=True, )
    title = models.ForeignKey(
        Title,
        blank=True, null=True,
        related_name='contacts',
        verbose_name=_("Title"),
    )
    first_name = models.CharField(
        max_length=100,
        verbose_name=_("First Name"),
    )
    last_name = models.CharField(
        max_length=100,
        verbose_name=_("Last Name"),
    )
    birth_date = models.DateField(
        null=True, blank=True,
        verbose_name=_("Birth Date"),
    )
    ssn = models.CharField(
        max_length=16,
        blank=True, null=True,
        db_index=True,
        verbose_name=_("SSN"),
    )
    organization = models.CharField(
        max_length=100,
        null=True, blank=True,
        verbose_name=_("Organization"),
    )
    origin = models.CharField(
        max_length=100,
        null=True, blank=True,
        verbose_name=_("Origin"),
    )
    role = models.ForeignKey(
        Role,
        blank=True, null=True,
        related_name='contacts',
        verbose_name=_("Role"),
    )
    url = models.URLField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("URL"),
    )
    source = models.CharField(
        max_length=100,
        null=True, blank=True,
        verbose_name=_("Source"),
    )
    category = models.ForeignKey(
        Category,
        blank=True, null=True,
        related_name='contacts',
        verbose_name=_("Category"),
    )
    groups = models.ManyToManyField(
        Group,
        related_name="contacts",
        verbose_name=_("Groups"),
    )
    first_group = models.ForeignKey(
        Group,
        blank=True, null=True,
        related_name='contacts_as_first_group',
        verbose_name=_("First group"),
    )
    interests = models.ManyToManyField(
        Interest,
        blank=True, null=True,
        related_name="contacts",
        verbose_name=_("Interests"),
    )
    products = models.ManyToManyField(
        Product,
        blank=True, null=True,
        related_name="contacts",
        verbose_name=_("Products"),
    )
    network = models.ForeignKey(
        Network,
        blank=True, null=True,
        related_name="contacts",
        verbose_name=_("Networks"),
    )
    # inizio newsletter
    date_subscription = models.DateTimeField(
        verbose_name=_("Date subscription"),
        blank=True, null=True
    )
    date_unsubscription = models.DateTimeField(
        verbose_name=_("Date unsubscription"),
        blank=True, null=True
    )
    # privacy
    date_last_modify_privacy = models.DateTimeField(
        verbose_name=_("Date last modify privacy"),
        blank=True, null=True
    )
    is_test_contact = models.BooleanField(
        default=False,
        verbose_name=_('test'),
    )
    # fine newsletter
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("Note"),
    )
    # campi portati da 'profilo' - start
    auth_trattamento_dati = models.BooleanField(
        _("Auth T.D."),
        help_text=_("Autorizzazione al trattamento dati"),
        default=False
    )
    auth_materiale_commerciale = models.NullBooleanField(
        _("Auth M.C."),
        help_text=_("Autorizzazione invio materiale commerciale"),
        blank=True, null=True, default=None
    )
    auth_societa_terze = models.BooleanField(
        _("Auth S.T."),
        help_text=_("Vendita dei dati a terzi"),
        default=False
    )
    auth_premium = models.BooleanField(
        _("Auth Premium"),
        help_text=_("Autorizzazione Premium"),
        default=False
    )
    # campi aggiunta con nuova registrazione light
    auth_profilazione = models.BooleanField(
        _("Auth Profilazione"),
        help_text=_("Autorizzazione alla profilazione"),
        default=False
    )
    # campi portati da 'profilo' - end
    # autorizzazione preliminare per invio mail
    bonifica = models.BooleanField(
        _("Bonifica"),
        default=False
    )
    user = models.OneToOneField(
        User,
        blank=True, null=True,
        related_name="contact",
        verbose_name=_("User"),
    )
    date_last_modify_for_mailup = models.DateTimeField(
        verbose_name=_('date last modify for mailup'),
        blank=True, null=True,
    )
    date_last_modify_groups_for_mailup = models.DateTimeField(
        verbose_name=_('date last modify groups for mailup'),
        blank=True, null=True,
    )
    mailup_recipient_id = models.IntegerField(
        null=True, blank=True,
        verbose_name=_("mailup recipient id")
    )
    trace_request_plugin = models.ForeignKey(
        "tracerequest.TraceRequestSettings",
        blank=True, null=True,
        on_delete=models.SET_NULL,
        related_name='contacts',
        verbose_name=_('trace request plugin'),
    )

    @property
    def premium_url(self):
        if self.premium_urls.all().count():
            return self.premium_urls.first().premium_url
        return None

    class Meta:
        verbose_name = _("Contact")
        verbose_name_plural = _("Contacts")
        ordering = ('last_name', 'first_name')
        permissions = (
            ('can_list_contact', 'Can list Contact'),
        )

    def __unicode__(self):
        return u"%s %s" % (self.last_name, self.first_name)

    def save(self, auto_now_date_privacy=True, *args, **kwargs):
        """
        Quando viene aggiunto (o rimosso) un gruppo, viene inviato un segnale
        acomea.contact.signals.contact_group_signal per gestire l'inserimento dei ContactGroup e del
        Contact.first_group (m2m_changed.connect(contact_group_signal, sender=Contact.groups.through)).
        Questa gestione è stata inserita nel segnale perchè i gruppi possono essere
        aggiunti al Contact anche senza passare dal save().
        """
        if auto_now_date_privacy:
            self.set_date_last_modify_privacy()
        self.set_privacy_log()

        # Controllo se il contatto ha bisogno di essere modificato in mailup
        now = datetime.datetime.utcnow().replace(tzinfo=utc)
        if self.is_changed():
            self.date_last_modify_for_mailup = now
        super(Contact, self).save(*args, **kwargs)

    def get_verified_first_group(self):
        first_contact_group = self.get_first_group_from_contact_group()

        # Contatto senza gruppi importanti associati (No lead e utenti_ar)
        if first_contact_group is None:
            return True, self.first_group, None

        if first_contact_group == self.first_group:
            return True, self.first_group, None

        return False, self.first_group, first_contact_group

    def get_first_group_from_contact_group(self):

        queryset = ContactGroup.objects.filter(contact=self)
        queryset = queryset.exclude(group_id__in=settings.ACOMEA_CONTACT_BASE_GROUPS)
        first_contact_groups = queryset.order_by('date_create')

        if first_contact_groups:
            return first_contact_groups[0].group

        return None

    def is_changed(self):
        """
        Controlla se sono cambiati i valori del contacts per aggiornare la data di mailup.
        Il controllo sull'aggiunta o la rimozione di un gruppo verrà fatta nel
            acomea.contact.signals.contact_group_signal
        """
        if self.id:
            # Check if contact privacy fields are changed
            old_contact = Contact.objects.get(id=self.id)
            if (
                self.first_name != old_contact.first_name or
                self.last_name != old_contact.last_name or
                self.bonifica != old_contact.bonifica or
                self.auth_trattamento_dati != old_contact.auth_trattamento_dati or
                self.auth_materiale_commerciale != old_contact.auth_materiale_commerciale or
                self.auth_profilazione != old_contact.auth_profilazione or
                self.date_unsubscription != old_contact.date_unsubscription or
                self.get_default_email() != old_contact.get_default_email()
                    # or contact_groups_ids != old_contact_groups_ids
            ):
                return True
            return False
        return True

    def set_date_last_modify_privacy(self):
        # INSERIMENTO
        if not self.id:
            self.date_last_modify_privacy = datetime.datetime.now()
        # MODIFICA
        else:
            old_contact = Contact.objects.get(id=self.id)
            if (
                old_contact.auth_trattamento_dati != self.auth_trattamento_dati or
                old_contact.auth_profilazione != self.auth_profilazione  or
                old_contact.auth_materiale_commerciale != self.auth_materiale_commerciale or
                old_contact.auth_societa_terze != self.auth_societa_terze
            ):
                self.date_last_modify_privacy = datetime.datetime.now()

    def set_privacy_log(self):
        try:
            # INSERIMENTO
            ip_address = None
            url = None
            last_modifier = None

            request = get_request()
            if request:
                last_modifier = get_current_user().username
                url = request.get_full_path()
                ip_address = request.META.get('REMOTE_ADDR')

            if not self.id:
                try:
                    default_email = self.emails.get(default=True)
                except:
                    default_email = ""

                text_log = """---;%s;%s %s;%s;-;-;-;%s;%s;%s;%s;%s;%s;""" % (
                    default_email,
                    self.last_name,
                    self.first_name,
                    last_modifier,
                    self.auth_trattamento_dati,
                    self.auth_materiale_commerciale,
                    self.auth_societa_terze,
                    self.auth_profilazione,
                    ip_address,
                    url
                )
                privacy_log.info(text_log)
            # MODIFICA
            else:
                try:
                    default_email = self.emails.get(default=True)
                except:
                    default_email = None

                old_contact = Contact.objects.get(id=self.id)
                if (
                    old_contact.auth_trattamento_dati != self.auth_trattamento_dati or
                    old_contact.auth_materiale_commerciale != self.auth_materiale_commerciale or
                    old_contact.auth_societa_terze != self.auth_societa_terze or
                    old_contact.auth_profilazione != self.auth_profilazione
                ):
                    text_log = "%s;%s;%s %s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;%s;" % (
                        self.id, default_email, self.last_name, self.first_name,
                        last_modifier,
                        old_contact.auth_trattamento_dati, old_contact.auth_materiale_commerciale,
                        old_contact.auth_societa_terze, old_contact.auth_profilazione,
                        self.auth_trattamento_dati, self.auth_materiale_commerciale, self.auth_societa_terze,
                        self.auth_profilazione,
                        ip_address, url
                    )
                    privacy_log.info(text_log)
        except Exception, e:
            text_log = "ATTENZIONE!!!: 'acomea.contact.models.set_privacy_log' - errore: %s - id: %s" % (
                e.message, self.id
            )
            privacy_log.error(text_log)

    def get_list_emails(self):
        list_emails = []
        for email in self.emails.filter(status=1).order_by("-default", "email"):
            text_email = email.email
            if email.default:
                text_email = u'<span style="{}" title="{}">{}&nbsp;{}</span>'.format(
                    'white-space: nowrap;',
                    _('Default email'),
                    email.email,
                    '<img src="/static/admin/img/icon-yes.gif" alt="True" style="margin-top:4px">'
                )
            list_emails.append(text_email)
        return list_emails

    def get_string_emails(self):
        return "<br>".join(self.get_list_emails())
    get_string_emails.allow_tags = True
    get_string_emails.short_description = _("Emails")

    def get_default_email(self):
        default_email = self.emails.filter(status=1, default=True)
        if default_email:
            return default_email[0].email
        return ''

    def get_first_name_title(self):
        return self.first_name.title()
    get_first_name_title.short_description = _("first_name")

    def get_last_name_title(self):
        return self.last_name.title()
    get_last_name_title.short_description = _("Last name")

    def get_email1(self):
        emails = self.emails.filter(status=1)
        try:
            return emails[1]
        except:
            return ''

    def get_email2(self):
        emails = self.emails.filter(status=1)
        try:
            return emails[2]
        except:
            return ''

    def get_list_groups(self):
        list_groups = []
        for group in self.groups.filter(status=1).order_by("name"):
            list_groups.append(group.name)
        return list_groups

    def get_string_groups(self):
        return ", ".join(self.get_list_groups())

    get_string_groups.short_description = _("Groups")

    def get_list_group_ids(self):
        list_groups = []
        for group in self.groups.filter(status=1).order_by("id"):
            list_groups.append(group.id)
        return list_groups

    def get_string_group_ids(self):
        return ", ".join(str(x) for x in self.get_list_group_ids())

    get_string_group_ids.short_description = _("Group ids")

    def get_list_phones(self):
        list_phones = []
        for phone in self.phones.filter(status=1).order_by("phone"):
            list_phones.append(phone.phone)
        return list_phones

    def get_string_phones(self):
        return ", ".join(self.get_list_phones())

    get_string_phones.short_description = _("Phones")

    def get_list_interests(self):
        list_interests = []
        for interest in self.interests.filter(status=1).order_by("name"):
            list_interests.append(interest.name)
        return list_interests

    def get_string_interests(self):
        return ", ".join(self.get_list_interests())

    get_string_interests.short_description = _("Interests")

    def get_list_interest_ids(self):
        list_interests = []
        for interest in self.interests.filter(status=1).order_by("id"):
            list_interests.append(interest.id)
        return list_interests

    def get_string_interest_ids(self):
        return ", ".join(str(x) for x in self.get_list_interest_ids())

    get_string_interest_ids.short_description = _("List interest ids")

    def get_list_products(self):
        list_products = []
        for product in self.products.filter(status=1).order_by("name"):
            list_products.append(product.name)
        return list_products

    def get_string_products(self):
        return ", ".join(self.get_list_products())

    get_string_products.short_description = _("Products")

    def get_list_product_ids(self):
        list_products = []
        for product in self.products.filter(status=1).order_by("id"):
            list_products.append(product.id)
        return list_products

    def get_string_product_ids(self):
        return ", ".join(str(x) for x in self.get_list_product_ids())

    get_string_product_ids.short_description = _("Product ids")

    def get_first_address(self):
        address_type = address_name = number = zip_code = None
        locality = province = region = None
        address_scope_id = country_id = address_name = None
        number = zip_code = locality = province = region = None
        addresses = self.addresses.filter(status=1)

        if addresses:
            address = addresses[0]

            address_type = address.address_type
            address_name = address.name
            number = address.number
            zip_code = address.zip_code
            locality = address.locality
            province = address.province
            region = address.region

            if address.address_scopes.all():
                address_scope_id = address.address_scopes.all()[0].id
            if address.country:
                country_id = address.country.id

        return {
            'get_address_scope_id': address_scope_id, 'get_address_type': address_type,
            'get_address_name': address_name, 'get_address_number': number, 'get_address_zip_code': zip_code,
            'get_address_locality': locality, 'get_address_province': province,
            'get_address_region': region, 'get_address_country_id': country_id
        }

    @property
    def can_create_message(self):
        if not self.bonifica:
            return False, _("contatto non bonificato")
        if not self.auth_trattamento_dati:
            return False, _("autorizzazione trattamento dati negata")
        if not self.auth_materiale_commerciale:
            return False, _("autorizzazione invio materiale commerciale negata")
        return True, "OK"

    @property
    def can_create_message_no_bonifica(self):
        if not self.auth_trattamento_dati:
            return False, _("autorizzazione trattamento dati negata")
        if not self.auth_materiale_commerciale:
            return False, _("autorizzazione invio materiale commerciale negata")
        return True, "OK"

    @property
    def can_create_message_risparmiamocelo(self):
        if not self.bonifica:
            return False, _("contatto non bonificato")
        if not self.auth_trattamento_dati:
            return False, _("autorizzazione trattamento dati negata")
        return True, "OK"

    def create_premium_url(self, premium_url):
        from acomea.premium.models import PremiumUrl
        return PremiumUrl.objects.create(contact=self, premium_url=premium_url)

    def obfuscate(self, to_save=True):
        self.status = 0
        self.title = None
        self.role = None
        self.birth_date = None
        self.category = None
        self.first_group = None
        self.network = None
        self.auth_trattamento_dati = False
        self.auth_materiale_commerciale = False
        self.auth_societa_terze = False
        self.auth_premium = False
        self.auth_profilazione = False
        self.first_name = 'deleted'
        self.last_name = 'deleted'
        if self.ssn:
            self.ssn = 'deleted'
        if self.organization:
            self.organization = 'deleted'
        if self.origin:
            self.origin = 'deleted'
        if self.url:
            self.url = 'deleted'
        if to_save:
            self.save()
        
        for (index, email) in enumerate(self.emails.all()):
            if index > 0:
                email.email = 'deleted_{}_{}@deleted.com'.format(self.user.pk, index)
            else:
                email.email = 'deleted_{}@deleted.com'.format(self.user.pk)
            email.save()

        return self

    def get_reload_from_db(self):
        """
        Return instance from db by pk
        """
        return self.__class__.objects.get(pk=self.pk)

    def assign_campaign_source(self, campaign, source):
        contact_campaign_source = ContactCampaignSource()
        try:
            contact_campaign_source.campaign = Campaign.objects.filter_by_slug_and_publicated(campaign).get()
        except:
            pass
        try:
            contact_campaign_source.source = Source.objects.filter_by_slug_and_publicated(source).get()
        except:
            pass

        if contact_campaign_source.campaign or contact_campaign_source.source:
            self.contacts_campaign_source.add(contact_campaign_source)


m2m_changed.connect(contact_group_signal, sender=Contact.groups.through)
post_delete.connect(delete_mailup_recipient, sender=Contact)


class Email(StatusModel, OrderedModel, DateModel, UserModel, DefaultModel):
    contact = models.ForeignKey(
        Contact,
        related_name="emails",
        verbose_name=_("Contact")
    )
    email = models.EmailField(
        max_length=100,
        verbose_name=_("Email"),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("Note"),
    )

    class Meta:
        verbose_name = _("Email")
        verbose_name_plural = _("Emails")
        ordering = ('email',)
        unique_together = (('email', 'contact'),)
        permissions = (
            ('can_list_email', 'Can list Email'),
        )

    def save(self, *args, **kwargs):
        # Questo viene fatto anche lato postgresq da un trigger
        # perché scrivono anche altri...
        self.email = self.email.lower()
        super(Email, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"%s" % self.email


class PhoneCategory(StatusModel, OrderedModel, DateModel, UserModel):
    """
    Stores phone categories
    EXAMPLE: 'casa', 'ufficio', 'azienda'
    """
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )

    class Meta:
        verbose_name = _("Phone category")
        verbose_name_plural = _("Phone categories")
        ordering = ('name',)
        permissions = (
            ('can_list_phonecategory', 'Can list Phone Category'),
        )

    def __unicode__(self):
        return u"%s" % self.name


class Phone(StatusModel, OrderedModel, DateModel, UserModel, DefaultModel):
    contact = models.ForeignKey(
        Contact,
        related_name="phones",
        verbose_name=_("Contact")
    )
    phone = models.CharField(
        max_length=100,
        blank=True,
        verbose_name=_("Phone"),
    )
    category = models.ForeignKey(
        PhoneCategory,
        related_name="phones",
        verbose_name=_("Category"),
        blank=True, null=True,
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("Note"),
    )

    class Meta:
        verbose_name = _("Phone")
        verbose_name_plural = _("Phones")
        ordering = ('category__name',)
        permissions = (
            ('can_list_phone', 'Can list Phone'),
        )

    def __unicode__(self):
        if self.category:
            return u"%s: %s" % (self.category.name, self.phone)
        return u"%s" % self.phone


class CategoryProfile(StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )

    class Meta:
        verbose_name = _("Profile category")
        verbose_name_plural = _("Profile categories")
        ordering = ('name',)
        permissions = (
            ('can_list_categoryprofile', 'Can list Category Profile'),
        )

    def __unicode__(self):
        return unicode(self.name)


class WebProfile(StatusModel, OrderedModel, DateModel, UserModel):
    contact = models.ForeignKey(
        Contact,
        related_name="profiles",
        verbose_name=_("Contact")
    )
    name = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )
    category = models.ForeignKey(
        CategoryProfile,
        related_name="profiles",
        verbose_name=_("Category")
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("Note"),
    )

    class Meta:
        verbose_name = _("Profile")
        verbose_name_plural = _("Profiles")
        ordering = ('name',)
        permissions = (
            ('can_list_webprofile', 'Can list Web Profile'),
        )

    def __unicode__(self):
        return unicode(self.name)


#ADDRESS_TYPE_CHOICES = (
#    ('corso', _(u'Corso')),
#    ('largo', _(u'Largo')),
#    ('localita', _(u'Località')),
#    ('piazza', _(u'Piazza')),
#    ('strada', _(u'Strada/Stretto')),
#    ('traversa', _(u'Traversa')),
#    ('via', _(u'Via')),
#    ('viale', _(u'Viale')),
#    ('vicolo', _(u'Vicolo/Vico')),
#    ('circonvallazione', _(u'Circonvallazione')),
#    ('frazione', _(u'Frazione')),
#    ('rione', _(u'Rione')),
#    ('salita', _(u'Salita')),
#    ('contrada', _(u'Contrada')),
#    ('statale', _(u'Statale')),
#    ('autostrada', _(u'Autostrada')),
#    ('cascina', _(u'Cascina')),
#    ('parco', _(u'Parco')),
#    ('spalto', _(u'Spalto')),
#    ('tangenziale', _(u'Tangenziale')),
#    ('zona industriale', _(u'Zona industriale')),
#    ('borgo', _(u'Borgo')),
#    ('galleria', _(u'Galleria')),
#    ('piazzale', _(u'Piazzale')),
#    ('corte', _(u'Corte')),
#    ('centro', _(u'Centro')),
#    ('foro', _(u'Foro')),
#    ('lungo', _(u'Lungo/Lungomare')),
#    ('calle', _(u'Calle')),
#    ('riviera', _(u'Riviera')),
#    ('villaggio', _(u'Villaggio')),
#    ('passaggio', _(u'Passaggio')),
#    ('borgata', _(u'Borgata')),
#    ('lungolago', _(u'Lungolago')),
#)

class AddressScope(StatusModel, OrderedModel, DateModel, UserModel):
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )

    class Meta:
        verbose_name = _("Address scope")
        verbose_name_plural = _("Address scopes")
        ordering = ('name',)
        permissions = (
            ('can_list_addressscope', 'Can list Address Scope'),
        )

    def __unicode__(self):
        return unicode(self.name)


class Address(StatusModel, OrderedModel, DateModel, UserModel):
    contact = models.ForeignKey(
        Contact,
        related_name="addresses",
        verbose_name=_("Contact")
    )
    address_scopes = models.ManyToManyField(
        AddressScope,
        db_index=True,
        related_name="addresses",
        verbose_name=_("Address scopes"),
    )
    address_type = models.CharField(
        max_length=20,
        default='via',
        choices=settings.CONTACT_ADDRESS_TYPE_CHOICES,
        db_index=True,
        verbose_name=_("Address type"),
        blank=True, null=True
    )
    name = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("Name"),
    )
    number = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("Number"),
        db_column="anumber"  # number in alcuni db (come oracle) e' una parola
        # riservata, quindi mettiamo come colonna del db
        # anumber per avere la piena compatibilita
    )
    zip_code = models.CharField(
        max_length=20,
        db_index=True,
        verbose_name=_("Zip code"),
        blank=True, null=True,
    )
    locality = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("Locality"),
        blank=True, null=True,
    )
    ## FIXME da sistemare (rifNazione, rifProvincia)
    province = models.CharField(
        max_length=2,
        db_index=True,
        verbose_name=_("Province"),
        blank=True, null=True,
    )
    region = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("Region"),
        blank=True, null=True,
    )
    country = models.ForeignKey(
        Country,
        db_index=True,
        default=108,  # id of Italy Country
        verbose_name=_("Country"),
    )

    def get_string_scopes(self):
        list_scopes = ""
        scopes = self.address_scopes.all()
        names = [scope.name for scope in scopes]
        return ", ".join(names)

    get_string_scopes.short_description = _("Scopes")

    class Meta:
        verbose_name = _("Address")
        verbose_name_plural = _("Addresses")
        permissions = (
            ('can_list_address', 'Can list Address'),
        )

    def __unicode__(self):
        return u"""
            %(address_type)s %(name)s, %(number)s - %(locality)s
            (%(province)s) - %(country)s
        """ % {
            'address_type': self.get_address_type_display(), 'name': self.name,
            'number': self.number, 'locality': self.locality, 'province': self.province,
            'country': self.country
        }


class AcomeaEmailObject(DateModel, UserModel):
    email_object = models.OneToOneField(
        EmailObject,
        related_name="acomea_email_object"
    )
    selected_groups = models.ManyToManyField(
        Group,
        null=True, blank=True,
        related_name="acomea_email_objects",
        verbose_name=_("Selected Groups"),
    )
    date_create_newsletter = models.DateTimeField(
        null=True, blank=True,
        db_index=True,
    )

    def get_list_groups(self):
        list_groups = []
        for group in self.selected_groups.all().order_by("name"):
            list_groups.append(group.name)
        return list_groups

    def get_string_groups(self):
        return ", ".join(self.get_list_groups())

    get_string_groups.short_description = _("Groups")

    def get_list_group_ids(self):
        list_groups = []
        for group in self.groups.all().order_by("id"):
            list_groups.append(group.id)
        return list_groups

    class Meta:
        verbose_name = _("AcomeA Email Object")
        verbose_name_plural = _("AcomeA Email Object")
        permissions = (
            ('can_list_acomeaemailobject', 'Can list Acomea Email Object'),
        )


class Campaign(StatusModel, OrderedModel, DateModel, UserModel):
    objects = managers.CampaignManager()

    title = models.CharField(
        max_length=100,
        verbose_name=_("title")
    )
    slug = models.SlugField(
        max_length=100,
        unique=True,
        verbose_name=_("slug"),
        help_text=_("slug to use in url")
    )
    description = models.TextField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("description")
    )
    date_publication_start = models.DateTimeField(
        verbose_name=_('date publication start')
    )
    date_publication_end = models.DateTimeField(
        verbose_name=_('date publication end'),
        blank=True, null=True
    )

    class Meta:
        verbose_name = _("Campaign")
        verbose_name_plural = _("Campaigns")
        ordering = ('ordering', 'title',)
        permissions = (
            ("list_campaign", "Can list campaign"),
            ("detail_campaign", "Can detail campaign"),
            ("disable_campaign", "Can disable campaign"),
        )

    def __unicode__(self):
        return u'{}'.format(self.slug)

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.title)
        super(Campaign, self).save(*args, **kwargs)

    # for dj 1.8
    # def clean(self):
    #     if self.date_publication_end <= self.date_publication_start:
    #         raise ValidationError(_('Date publication end must be greater than Date publication start'))
    #     super().clean()


class Source(StatusModel, OrderedModel, DateModel, UserModel):
    objects = managers.SourceManager()

    title = models.CharField(
        max_length=100,
        verbose_name=_("title")
    )
    slug = models.SlugField(
        max_length=100,
        unique=True,
        verbose_name=_("slug"),
        help_text=_("slug to use in url")
    )
    description = models.TextField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("description")
    )
    date_publication_start = models.DateTimeField(
        verbose_name=_('date publication start')
    )
    date_publication_end = models.DateTimeField(
        verbose_name=_('date publication end'),
        blank=True, null=True
    )

    class Meta:
        verbose_name = _("Source")
        verbose_name_plural = _("Sources")
        ordering = ('ordering', 'title',)
        permissions = (
            ("list_source", "Can list source"),
            ("detail_source", "Can detail source"),
            ("disable_source", "Can disable source"),
        )

    def __unicode__(self):
        return u'{}'.format(self.slug)

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify(self.title)
        super(Source, self).save(*args, **kwargs)

    # for dj 1.8
    # def clean(self):
    #     if self.date_publication_end <= self.date_publication_start:
    #         raise ValidationError(_('Date publication end must be greater than Date publication start'))
    #     super().clean()


class ContactCampaignSource(DateModel):
    """
    Questa classe potrebbe essere la through tra Contact Campaign e Source, ma non è stata impostata così
    per utilizzare nell'admin il filter_horizontal.
    Questo modello viene gestito solo durante la registrazione

    """
    contact = models.ForeignKey(
        Contact,
        unique=True,
        related_name="contacts_campaign_source",
        verbose_name=_("Contact")
    )
    campaign = models.ForeignKey(
        Campaign,
        blank=True, null=True,
        related_name="contacts_campaign_source",
        verbose_name=_("campaign"),
        on_delete=models.PROTECT
    )
    source = models.ForeignKey(
        Source,
        blank=True, null=True,
        related_name="contacts_campaign_source",
        verbose_name=_("source"),
        on_delete=models.PROTECT
    )

    class Meta:
        verbose_name = _("Contact campaign source")
        verbose_name_plural = _("Contact campaigns sources")
        permissions = (
            ("list_contactcampaignsource", "Can list contactcampaignsource"),
            ("detail_contactcampaignsource", "Can detail contactcampaignsource"),
            ("disable_contactcampaignsource", "Can disable contactcampaignsource"),
        )

    def __unicode__(self):
        str = u'{}'.format(getattr(self.contact, 'user', self.contact))
        if self.campaign:
            str = u'{}; campaign:{}'.format(str, self.campaign)
        if self.source:
            str = u'{}; source:{}'.format(str, self.source)
        return str

    def save(self, *args, **kwargs):
        if not self.campaign and not self.source:
            raise ValidationError(_('Indicare almeno una Campagna o una Sorgente'))
        super(ContactCampaignSource, self).save(*args, **kwargs)


if settings.CMS_ENABLED:
    import acomea.contact.cms.plugins
    import acomea.contact.cms.app
